package gov.nih.nlm.nls.lexCheck.install.Setup;

import java.util.*;
import java.io.*;
import java.lang.*;

import gov.nih.nlm.nls.lexCheck.install.utils.*;

public class CreateScripts
{
	/**
	* Method CreateExecScripts creates bourne, and batch scripts
	* for la, norm, luiNorm, and wordInd and puts them in 
	* the $LA_DIR/bin directory 
	*/
	public static void CreateExecScripts() throws Exception
	{
		// Determine the LA_DIR path
		String rootDir = InstallObject.GetRootDir();

		// Determine the CLASSPATH 
		String classPath = InstallObject.GetClassPath("LC_DIR");

		// Invoke java with the class 
		String pPath = "gov.nih.nlm.nls.lexCheck.Tools.";

		CreateExecScript(rootDir, classPath, pPath, "GenerateInflVars");
		CreateExecScript(rootDir, classPath, pPath, "LexCheck");
		CreateExecScript(rootDir, classPath, pPath, "LexCrossCheck");
		CreateExecScript(rootDir, classPath, pPath, "ToJavaObjectFromTextFile");
		CreateExecScript(rootDir, classPath, pPath, "ToJavaObjectFromXmlFile");
		CreateExecScript(rootDir, classPath, pPath, "ToXmlFromTextFile");
		CreateExecScript(rootDir, classPath, pPath, "ValidateContentFromTextFile");
		CreateExecScript(rootDir, classPath, pPath, "ValidateSyntaxFromTextFile");
	}

	private static void CreateExecScript(String pLA_DIR, String pClassPath, 
		String pPath, String pExecName) throws Exception
  	{
		String script = new String();
		String extension = "";

		// Determine the platform
    	if(U.isWindows() == true)
		{
			script = CreateWindowScript(pLA_DIR, pClassPath, pPath, pExecName);
			extension = ".bat";
    	} 
		else	// unix or linux
		{
			script = CreateUnixScript(pLA_DIR, pClassPath, pPath, pExecName);
		}
    
    	// Figure out the fileName of the script
    	String scriptFileName = pLA_DIR + U.FS + "bin" + U.FS + pExecName 
			+ extension;
    
		// Write this out to a file
    	try 
		{ 
      		PrintWriter fStream = new PrintWriter(
				new BufferedWriter(new FileWriter(scriptFileName)));
      		fStream.print(script);
			fStream.close();
			fStream = null;
    	} 
		catch (Exception e) 
		{
      		System.out.println("** Err: Not able to write to " 
				+ scriptFileName);
    	}
	}

	private static String CreateUnixScript(String pLA_DIR, String pClassPath, 
		String pPath, String pExecName) throws Exception
	{
		String commentStart = "# ";
		String BEGIN_VARIABLE = "${";
		String END_VARIABLE = "}";
		String javaCommand = "$JAVA ";
		String setCommand = "";
		String doubleQuote = "\"";

		String rootDir = InstallObject.GetRootDir();
		StringBuffer buff = new StringBuffer();

   		// Start a bourne shell up
		buff.append("#!/bin/sh -f"); 
		buff.append(U.NL);

		// Add some comments to what this script is 
		buff.append(GetScriptHeader(commentStart, pExecName));
    
   		// Setup the variable
		buff.append("JAVA=");
		buff.append("java");
		buff.append(U.NL);
		buff.append(U.NL);

		// Set the ROOT_DIR 
		buff.append(GetRootDir(commentStart, setCommand, pLA_DIR));
		
		// Set the classpath 
		buff.append(GetClasspath(commentStart, setCommand, doubleQuote, 
			pClassPath)); 

		buff.append("export LA_DIR JAVA CLASSPATH");
		buff.append(U.NL);
		buff.append(U.NL);
    
		// Run the command 
		buff.append(GetRunCommand(commentStart, javaCommand, BEGIN_VARIABLE,
			END_VARIABLE, pPath, pExecName)); 

		return buff.toString();
	}

	private static String CreateWindowScript(String pLA_DIR, String pClassPath, 
		String pPath, String pExecName) throws Exception
  	{
		String commentStart = "@REM # ";
		String BEGIN_VARIABLE = "%";
		String END_VARIABLE = "%";
		String javaCommand = "@java ";
		String setCommand = "@set ";
		String doubleQuote = "\"";

		String rootDir = InstallObject.GetRootDir();
		StringBuffer buff = new StringBuffer();

    	// Add some comments to what this script is 
		buff.append(GetScriptHeader(commentStart, pExecName));
    
		// Set the LA_DIR 
		buff.append(GetRootDir(commentStart, setCommand, pLA_DIR)); 
		
		// Set the classpath 
		buff.append(GetClasspath(commentStart, setCommand, doubleQuote,
			pClassPath)); 
    
		// Run the command 
		buff.append(GetRunCommand(commentStart, javaCommand, BEGIN_VARIABLE,
			END_VARIABLE, pPath, pExecName));

		return buff.toString();
	}

	private static String GetScriptHeader(String commentStart, String pExecName)
	{
		StringBuffer buff = new StringBuffer();

		// Add some comments to what this script is 
		buff.append(commentStart);
		buff.append("---------------------------------");
		buff.append(U.NL);
		buff.append(commentStart);
		buff.append("This is the batch file that sets");
		buff.append(U.NL);
		buff.append(commentStart);
		buff.append("the initial classpath, and runs");
		buff.append(U.NL);
		buff.append(commentStart);
		buff.append("the Java "); 
		buff.append(pExecName);
		buff.append(" Program."); 
		buff.append(U.NL);
		buff.append(commentStart);
		buff.append("---------------------------------");
		buff.append(U.NL);

		return buff.toString();
	}

	private static String GetRootDir(String commentStart, String setCommand,
		String pLA_DIR)
	{
		StringBuffer buff = new StringBuffer();

		// Set the ROOT_DIR 
		buff.append(commentStart); 
		buff.append("------------------"); 
		buff.append(U.NL);
		buff.append(commentStart); 
		buff.append("Set the LC_DIR"); 
		buff.append(U.NL);
		buff.append(commentStart); 
		buff.append("------------------"); 
		buff.append(U.NL);
		buff.append(setCommand);
		buff.append("LC_DIR=");
		buff.append(pLA_DIR);
		buff.append(U.NL);
		buff.append(U.NL);

		return buff.toString();
	}

	private static String GetClasspath(String commentStart, String setCommand,
		String doubleQuote, String pClassPath)
	{
		StringBuffer buff = new StringBuffer();

		// Set the classpath 
		buff.append(commentStart); 
		buff.append("------------------"); 
		buff.append(U.NL);
		buff.append(commentStart); 
		buff.append("Set the classpath"); 
		buff.append(U.NL);
		buff.append(commentStart); 
		buff.append("------------------"); 
		buff.append(U.NL);
		buff.append(setCommand);
		buff.append("CLASSPATH=");
		buff.append(doubleQuote);
		buff.append(pClassPath);
		buff.append(doubleQuote);
		buff.append(U.NL);
		buff.append(U.NL);

		return buff.toString();
	}

	private static String GetRunCommand(String commentStart, String javaCommand,
		String BEGIN_VARIABLE, String END_VARIABLE, String pPath, 
		String pExecName)
	{
		StringBuffer buff = new StringBuffer();

		// Run the command 
		buff.append(commentStart); 
		buff.append("------------------"); 
		buff.append(U.NL);
		buff.append(commentStart); 
		buff.append("Run the command "); 
		buff.append(U.NL);
		buff.append(commentStart); 
		buff.append("------------------"); 
		buff.append(U.NL);
		buff.append(javaCommand);
		buff.append(" -classpath ");
		buff.append(BEGIN_VARIABLE); 
		buff.append("CLASSPATH");
		buff.append(END_VARIABLE); 
		buff.append(" ");
		buff.append(pPath);
		buff.append(pExecName);
		buff.append(" ");
		buff.append(BEGIN_VARIABLE);
		buff.append("*");
		buff.append(END_VARIABLE); 
		buff.append(U.NL);

		return buff.toString();
	}
}
