package gov.nih.nlm.nls.lexCheck.install.Setup;

import java.util.*;
import java.io.*;
import java.lang.*;

import gov.nih.nlm.nls.lexCheck.install.utils.*;

import gov.nih.nlm.nls.lexCheck.Api.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;

public class Verify
{
	/**
	* @return boolean (True if a call to la matches it's expected output)
	*/
	public static boolean Verify() throws Exception 
  	{
    	boolean flag = false;
		String rootDir = InstallObject.GetRootDir();
    
    	String configFileToUse = rootDir + U.FS + "data" + U.FS + "config" 
			+ U.FS + "lexCheck.properties";
    
		// Send Verifying information 
		String msgDir = InstallObject.GetMsgDir();
		Lib.DisplayMessageFile(msgDir, "Verify.txt");

		// assign gold
		StringBuffer gold = new StringBuffer();
		gold.append("{base=see"); gold.append(U.NL);
		gold.append("entry=E0055007"); gold.append(U.NL);
		gold.append("\tcat=verb"); gold.append(U.NL);
		gold.append("\tvariants=irreg|see|sees|saw|seen|seeing|"); 
		gold.append(U.NL);
		gold.append("\tintran"); gold.append(U.NL);
		gold.append("\ttran=np"); gold.append(U.NL);
		gold.append("\ttran=pphr(about,ingcomp:arbc)"); gold.append(U.NL);
		gold.append("\ttran=pphr(about,np)"); gold.append(U.NL);
		gold.append("\ttran=whfincomp"); gold.append(U.NL);
		gold.append("\ttran=whinfcomp:arbc"); gold.append(U.NL);
		gold.append("\tcplxtran=np,advbl"); gold.append(U.NL);
		gold.append("\tcplxtran=np,binfcomp:objr"); gold.append(U.NL);
		gold.append("\tcplxtran=np,edcomp:objr"); gold.append(U.NL);
		gold.append("\tcplxtran=np,infcomp:objr"); gold.append(U.NL);
		gold.append("\tcplxtran=np,ingcomp:objr"); gold.append(U.NL);
		gold.append("}"); gold.append(U.NL);

		// Switch the classpath before invoking this command 
		String classPath = InstallObject.GetClassPath();
		System.setProperty("java.class.path", classPath);

		LexRecord result = ToJavaObjApi.ToJavaObjFromText(gold.toString());
		String response = result.GetText();

		// convert to Unix Line Separator for different DBs
		String pureGold = Convert.ToUnixLineSeparator(gold.toString());
		String pureResponse = Convert.ToUnixLineSeparator(response);

    	// Compare the results
    	if(pureResponse.compareTo(pureGold.toString()) == 0) 
		{
			flag = true;
    	}
    	else 
		{
			System.out.println("+-----------------------------------+");
			System.out.println("| Something is not quite right.     |");
			System.out.println("| The test produced :               |");
			System.out.println("+-----------------------------------+");
			System.out.println( "+" + response + "+" );
			System.out.println( "+ size: " + pureResponse.length() + "+" );
			System.out.println("+-----------------------------------+");
			  
			System.out.println("+-----------------------------------+");
			System.out.println("| But the test should have produced:|");
			System.out.println("+-----------------------------------+");
			System.out.println( "+" + pureGold.toString()  + "+" );
			System.out.println( "+ size: " + pureGold.toString().length() + "+" );
			System.out.println("+-----------------------------------+");
		}	

		return flag;
	}

	// private method
	private static String Replace(String inStr, String src, String tar)
	{
		String outStr = inStr;
		int tarSize = tar.length();

		int index = outStr.indexOf(src);
		while(index >= 0)
		{
			outStr = outStr.substring(0, index) + tar
				+ outStr.substring(index+1);
			index = outStr.indexOf(src, index+tarSize);
		}

		return outStr;
	}
}
