package gov.nih.nlm.nls.lexCheck.Api;
import com.ibm.xml.parser.Parser;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Hashtable;
import org.w3c.dom.*;
import java.util.*;
import java.io.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Gram.*;
/*****************************************************************************
* This class provides two Java Apis for transfer  <ul>
* <li>lexical records in Xml format (the input must be a file)
* <li>lexical records in text format (the input is a String)
* into Java Object.
*
* <p><b>History:</b>
* <ul>
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2013
****************************************************************************/
public class ToJavaObjApi 
{
    /**
    * A method to read lexical records from a XMl file and then transfer to 
    * a Vector of Java Lexical objects. 
    *
    * @param xmlFileName  the file name of the lexical records in Xml format
    *
    * @return a Vector of Java Lexical record objects.
    *
    * @deprecated as in 2006, replaced by ToJavaObjsFromXmlFile(string)
    */
    @Deprecated public static Vector<LexRecord> ToJavaObjs(String xmlFileName) 
        throws Exception
    {
        return ToJavaObjsFromXmlFile(xmlFileName);
    }
    /**
    * A method to read lexical records from a XMl file and then transfer to 
    * a Vector of Java Lexical objects.
    *
    * @param xmlFileName  the file name of the lexical records in Xml format
    *
    * @return a Vector of Java Lexical record objects.
    */
    public static Vector<LexRecord> ToJavaObjsFromXmlFile(String xmlFileName) 
        throws Exception
    {
        // Open specified file
        InputStream is = new FileInputStream(xmlFileName);
        // Start parsing
        Parser parser = new Parser(xmlFileName);
        Document doc = parser.readStream(is);
        // Check if there is errors, well-formed
        if(parser.getNumberOfErrors() > 0)
        {
            return null;
        }
        // go through document
        Vector<LexRecord> lexRecords = GetLexRecords(doc);
        return lexRecords;
    }
   /**
    * A method to read lexical records from a text formatted file and transfer 
    * into a Vector of Java Lexical objects.
    *
    * @param inFile  the file name of the lexical records in text format
    *
    * @return a Vector of Java Lexical record objects.
    */
    public static Vector<LexRecord> ToJavaObjsFromTextFile(String inFile) 
        throws Exception
    {
        CheckSt st = new CheckSt();     // upper level  Check state
        CheckSt catSt = new CheckSt(CheckSt.CHECK_CAT_ENTRY);
            // Category dependant Check state
        LineObject lineObject = new LineObject();
        int recordNum = 0;
        Vector<LexRecord> lexRecords = new Vector<LexRecord>();
        try
        {
            BufferedReader inReader = new BufferedReader(new InputStreamReader(
                new FileInputStream(inFile), "UTF-8"));
            while(lineObject != null)
            {
                // check if read in the next line
                if(lineObject.IsGoToNext() == true)
                {
                    lineObject.SetLine(inReader.readLine());
                    lineObject.IncreaseLineNum();
                }
                if(lineObject.GetLine() == null)
                {
                    break;
                }
                recordNum = 
                    CheckLine(st, catSt, lineObject, lexRecords, recordNum);
            }
            inReader.close();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return lexRecords;
    }
   /**
    * A method to transfer lexical records in text format (string) into a 
    * Vector of Java Lexical objects.
    *
    * @param text  the content of lexical records in text format
    *
    * @return a Vector of Java Lexical record objects.
    */
    public static Vector<LexRecord> ToJavaObjsFromText(String text) 
        throws Exception
    {
        CheckSt st = new CheckSt();     // upper level  Check state
        CheckSt catSt = new CheckSt(CheckSt.CHECK_CAT_ENTRY);
            // Category dependant Check state
        LineObject lineObject = new LineObject();
        // Convert to Unix line Separator for different platform
        String unixText = Convert.ToUnixLineSeparator(text);
        StringTokenizer buf =
            new StringTokenizer(unixText, Convert.UNIX_LINE_SEP);
        Vector<LexRecord> lexRecords = new Vector<LexRecord>();
        while(lineObject != null)
        {
            // check if read in the next line
            if(lineObject.IsGoToNext() == true)
            {
                if(buf.hasMoreTokens() == false)
                {
                    break;
                }
                lineObject.SetLine(buf.nextToken());
                lineObject.IncreaseLineNum();
            }
            
            CheckLine(st, catSt, lineObject, lexRecords);
        }
        return lexRecords;
    }
   /**
    * A method transfer an lexical record in text format into a Java Lexical 
    * object. If the input text include multiple records, only the last one 
    * will be returned.
    *
    * @param text  the content of the lexical record in text format
    *
    * @return a Java Lexical record object.
    */
    public static LexRecord ToJavaObjFromText(String text) throws Exception
    {
        ErrMsg.ResetErrMsg();
        CheckSt st = new CheckSt();     // upper level  Check state
        CheckSt catSt = new CheckSt(CheckSt.CHECK_CAT_ENTRY);
            // Category dependant Check state
        LineObject lineObject = new LineObject();
        // Convert to Unix line Separator for different platform
        String unixText = Convert.ToUnixLineSeparator(text);
        // get line by line and put into buf
        StringTokenizer buf =
            new StringTokenizer(unixText, Convert.UNIX_LINE_SEP);
        LexRecord lexRecord = new LexRecord();
        while(lineObject != null)
        {
            // check if read in the next line
            if(lineObject.IsGoToNext() == true)
            {
                if(buf.hasMoreTokens() == false)
                {
                    break;
                }
                lineObject.SetLine(buf.nextToken());
                lineObject.IncreaseLineNum();
            }
            lexRecord = CheckLine(st, catSt, lineObject);
        }
        return lexRecord;
    }
   /**
    * A method to check grammer of lexical records in text format (string).
    *
    * @param text  the content of the lexical records in text format
    *
    * @return true if no errors found and vise versa
    */
    public static boolean CheckRecordText(String text) throws Exception
    {
        ErrMsg.ResetErrMsg();
        CheckSt st = new CheckSt();     // upper level  Check state
        CheckSt catSt = new CheckSt(CheckSt.CHECK_CAT_ENTRY);
            // Category dependant Check state
        LineObject lineObject = new LineObject();
        // Convert to Unix line Separator for different platform
        String unixText = Convert.ToUnixLineSeparator(text);
        StringTokenizer buf =
            new StringTokenizer(unixText, Convert.UNIX_LINE_SEP);
        while(lineObject != null)
        {
            // check if read in the next line
            if(lineObject.IsGoToNext() == true)
            {
                // end of the file (text)
                if(buf.hasMoreTokens() == false)
                {
                    break;
                }
                lineObject.SetLine(buf.nextToken());
                lineObject.IncreaseLineNum();
            }
            if(CheckLine2(st, catSt, lineObject) == false)
            {
                return false;
            }
        }
        return true;
    }
    /**
    * A method to check grammer of lexical records from a flat file.
    *
    * @param inFile  the file name of the lexical records in text format
    *
    * @return -1 if erros found or the total number of records checked 
    */
    public static int CheckRecordsFromTextFile(String inFile) 
        throws Exception
	{
		String prepositionFile = new String();
		return CheckRecordsFromTextFile(inFile, prepositionFile);
	}
    /**
    * A method to check grammer of lexical records from a flat file.
    *
    * @param inFile  the file name of the lexical records in text format
    *
    * @return -1 if erros found or the total number of records checked 
    */
    public static int CheckRecordsFromTextFile(String inFile, 
		String prepositionFile) throws Exception
    {
		// set preposition file
		GlobalVars.SetPrepositionFile(prepositionFile);
		
        lexRecNum_ = 0;        // reset static variable
        CheckSt st = new CheckSt();     // upper level  Check state
        CheckSt catSt = new CheckSt(CheckSt.CHECK_CAT_ENTRY);
            // Category dependant Check state
        LineObject lineObject = new LineObject();
        BufferedReader inReader = new BufferedReader(new InputStreamReader(
            new FileInputStream(inFile), "UTF-8"));
        while(lineObject != null)
        {
            // check if read in the next line
            if(lineObject.IsGoToNext() == true)
            {
                String line = inReader.readLine();
                if(line == null)
                {
                    break;
                }
                //lineObject.SetLine(inReader.readLine());
                lineObject.SetLine(line);
                lineObject.IncreaseLineNum();
            }
            if(CheckLine2(st, catSt, lineObject) == false)
            {
                return -1;
            }
        }
        inReader.close();
        return lexRecNum_;
    }
    /**
    * This method return the error message when the first error happen. 
    *
    * @return the first error message of syntax check on a LexRecord
    */
    public static String GetErrMsg()
    {
        return ErrMsg.GetErrMsg();
    }
    /**
    * A unit test driver for ToJavaObjApi.
    */
    static public void main(String[] argv) 
    {
        String bat =
"{base=bat\nentry=E0012112\n\tcat=noun\n\tvariants=reg\nsignature=vanni\n}";
        String molt =
"{base=molt\nspelling_variant=moult\nentry=E0040723\n\tcat=noun\n\tvariants=reg\n\tacronym_of=test123\n\tabbreviation_of=test456\n\tacronym_of=test789\nsignature=vanni\n}\n{base=molt\nspelling_variant=moult\nentry=E0040724\n\tcat=verb\n\tvariants=reg\n\tintran\n\ttran=np\n\tnominalization=molting|noun|E0412675\nsignature=vanni\n}";
        if(argv.length == 0)
        {
            try
            {
                Vector<LexRecord> lexRecords = ToJavaObjsFromText(molt);
                for(int i = 0; i < lexRecords.size(); i++)
                {
                    LexRecord lexRecord = lexRecords.elementAt(i);
                    System.out.println(lexRecord.GetText());
                    //System.out.println(lexRecord.GetXml());
                    System.out.println("---------------------------------");
                }
                LexRecord lexRecord = ToJavaObjFromText(molt);
                System.out.println(lexRecord.GetText());
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
        else if(argv.length == 1)
        {
            try
            {
                Vector<LexRecord> lexRecords = ToJavaObjs(argv[0]);
                if(lexRecords.size() <= 0)
                {
                    System.exit(1);
                }
                else
                {
                    for(int i = 0; i < lexRecords.size(); i++)
                    {
                        LexRecord lexRecord = lexRecords.elementAt(i);
                        System.out.println(lexRecord.GetText());
                        System.out.println(lexRecord.GetXml());
                        System.out.println("---------------------------------");
                        InflVarsAndAgreements infls = 
                            new InflVarsAndAgreements(lexRecord);
                        Vector<InflVar> inflVars = infls.GetInflValues();
                        for(int j = 0; j < inflVars.size(); j++)
                        {
                            InflVar inflVar = inflVars.elementAt(j);
                            System.out.println(
                                "unInfl: " + inflVar.GetUnInfl()
                                + "; infl: " + inflVar.GetInflection() 
                                + "; var: " + inflVar.GetVar());
                        }
                    }
                }
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
        else // illegal input
        {
            System.err.println("*** Usage: java ToJavaObjApi <filename>");
            System.exit(1);
        }
    }
    /**
    * A method to transfer lexical records into a Vector of Java Lexical 
    * objects.
    *
    * @param node  the top node of DOM object.
    *
    * @return a Vector of Java Lexical record objects.
    */
    private static Vector<LexRecord> GetLexRecords(Node node) throws Exception
    {
        Vector<LexRecord> lexRecords = new Vector<LexRecord>();
        // Traverse all the children of the root element
        LexRecord lexRecord = null;
        CatEntry catEntry = null;
        // go through document
        for(Node records = node.getFirstChild(); 
            records != null; records = records.getNextSibling()) 
        {
            for(Node record = records.getFirstChild(); 
                record != null; record = record.getNextSibling()) 
            {
                if(record.getNodeType() == Node.ELEMENT_NODE)
                {
                    if(record.getNodeName().equals("lexRecord"))
                    {
                        lexRecord = new LexRecord();
                        GetLexRecord(record, lexRecord, catEntry);
                    }
                    // print out xml
                    //lexRecord.PrintText();
                    lexRecords.addElement(lexRecord);
                }
            }
        }
        return lexRecords;
    }
    private static void GetLexRecord(Node node, LexRecord lexRecord,
        CatEntry catEntry) throws Exception
    {
        // When child is an element
        if(node.getNodeType() == Node.ELEMENT_NODE)
        {
            if(node.getNodeName().equals("base"))
            {
                lexRecord.SetBase(getValue(node));
            }
            else if(node.getNodeName().equals("eui"))
            {
                lexRecord.SetEui(getValue(node));
            }
            else if(node.getNodeName().equals("cat")) {
                lexRecord.SetCat(getValue(node));
            }
            else if(node.getNodeName().equals("spellingVars"))
            {
                lexRecord.SetSpellingVar(getValue(node));
            }
            /***
            else if(node.getNodeName().equals("inflVars"))
            {
                // do nothing
            }
            ***/
            else if(node.getNodeName().equals("acronyms"))
            {
                lexRecord.GetAcronyms().addElement(getValue(node));
            }
            else if(node.getNodeName().equals("abbreviations"))
            {
                lexRecord.GetAbbreviations().addElement(getValue(node));
            }
            else if(node.getNodeName().equals("annotations"))
            {
                lexRecord.GetAnnotations().addElement(getValue(node));
            }
            else if(node.getNodeName().equals("signature"))
            {
                lexRecord.SetSignature(getValue(node));
            }
            // catEntry
            else if(node.getNodeName().equals("verbEntry"))
            {
                catEntry = getVerbValues(node);
                lexRecord.SetCatEntry(catEntry);
            }
            else if(node.getNodeName().equals("auxEntry"))
            {
                catEntry = getAuxValues(node);
                lexRecord.SetCatEntry(catEntry);
            }
            else if(node.getNodeName().equals("modalEntry"))
            {
                catEntry = getModalValues(node);
                lexRecord.SetCatEntry(catEntry);
            }
            else if(node.getNodeName().equals("nounEntry"))
            {
                catEntry = getNounValues(node);
                lexRecord.SetCatEntry(catEntry);
            }
            else if(node.getNodeName().equals("pronEntry"))
            {
                catEntry = getPronValues(node);
                lexRecord.SetCatEntry(catEntry);
            }
            else if(node.getNodeName().equals("adjEntry"))
            {
                catEntry = getAdjValues(node);
                lexRecord.SetCatEntry(catEntry);
            }
            else if(node.getNodeName().equals("advEntry"))
            {
                catEntry = getAdvValues(node);
                lexRecord.SetCatEntry(catEntry);
            }
            else if(node.getNodeName().equals("detEntry"))
            {
                catEntry = getDetValues(node);
                lexRecord.SetCatEntry(catEntry);
            }
        }
        // go through document
        for(Node child = node.getFirstChild();
            child != null; child = child.getNextSibling()) 
        {
            GetLexRecord(child, lexRecord, catEntry);    
        }
    }
    private static CatEntry getVerbValues(Node node)
    {
        CatEntry catEntry = new CatEntry("verb");
           // Visit all the child nodes
           for(Node ch = node.getFirstChild(); ch != null; ch=ch.getNextSibling())
        {
            // Recursively call if the child may have children
            if(ch instanceof Element) 
            {
                if(ch.getNodeName().equals("variants"))
                {
                    catEntry.GetVerbEntry().GetVariants().addElement(
                        getValue(ch));
                }
                else if(ch.getNodeName().equals("intran"))
                {
                    catEntry.GetVerbEntry().GetIntran().addElement(
                        getValue(ch));
                }
                else if(ch.getNodeName().equals("tran"))
                {
                    catEntry.GetVerbEntry().GetTran().addElement(
                        getValue(ch));
                }
                else if(ch.getNodeName().equals("ditran"))
                {
                    catEntry.GetVerbEntry().GetDitran().addElement(
                        getValue(ch));
                }
                else if(ch.getNodeName().equals("link"))
                {
                    catEntry.GetVerbEntry().GetLink().addElement(
                        getValue(ch));
                }
                else if(ch.getNodeName().equals("cplxtran"))
                {
                    catEntry.GetVerbEntry().GetCplxtran().addElement(
                        getValue(ch));
                }
                else if(ch.getNodeName().equals("nominalization"))
                {
                    catEntry.GetVerbEntry().GetNominalization().addElement(
                        getValue(ch));
                }
            }
        }
        return catEntry;
    }
    private static CatEntry getAuxValues(Node node)
    {
        CatEntry catEntry = new CatEntry("aux");
           // Visit all the child nodes
           for(Node ch = node.getFirstChild(); ch != null; ch=ch.getNextSibling())
        {
            // Recursively call if the child may have children
            if(ch instanceof Element) 
            {
                if(ch.getNodeName().equals("variant"))
                {
                    catEntry.GetAuxEntry().GetVariant().addElement(
                        getValue(ch));
                }
            } 
        }
        return catEntry;
    }
    private static CatEntry getModalValues(Node node)
    {
        CatEntry catEntry = new CatEntry("modal");
           // Visit all the child nodes
           for(Node ch = node.getFirstChild(); ch != null; ch=ch.getNextSibling())
        {
            // Recursively call if the child may have children
            if(ch instanceof Element) 
            {
                if(ch.getNodeName().equals("variant"))
                {
                    catEntry.GetModalEntry().GetVariant().addElement(
                        getValue(ch));
                }
            } 
        }
        return catEntry;
    }
    private static CatEntry getNounValues(Node node)
    {
        CatEntry catEntry = new CatEntry("noun");
           // Visit all the child nodes
           for(Node ch = node.getFirstChild(); ch != null; ch=ch.getNextSibling())
        {
            // Recursively call if the child may have children
            if(ch instanceof Element) 
            {
                if(ch.getNodeName().equals("variants"))
                {
                    catEntry.GetNounEntry().GetVariants().addElement(
                        getValue(ch));
                }
                else if(ch.getNodeName().equals("compl"))
                {
                    catEntry.GetNounEntry().GetCompl().addElement(
                        getValue(ch));
                }
                else if(ch.getNodeName().equals("nominalization"))
                {
                    catEntry.GetNounEntry().GetNominalization().addElement(
                        getValue(ch));
                }
                else if(ch.getNodeName().equals("tradeName"))
                {
                    catEntry.GetNounEntry().GetTradeName().addElement(
                        getValue(ch));
                }
                else if(ch.getNodeName().equals("trademark"))
                {
                    catEntry.GetNounEntry().SetTradeMark(true);
                }
                else if(ch.getNodeName().equals("proper"))
                {
                    catEntry.GetNounEntry().SetProper(true);
                }
            } 
        }
        return catEntry;
    }
    private static CatEntry getPronValues(Node node)
    {
        CatEntry catEntry = new CatEntry("pron");
           // Visit all the child nodes
           for(Node ch = node.getFirstChild(); ch != null; ch=ch.getNextSibling())
        {
            // Recursively call if the child may have children
            if(ch instanceof Element) 
            {
                if(ch.getNodeName().equals("variants"))
                {
                    catEntry.GetPronEntry().GetVariants().addElement(
                        getValue(ch));
                }
                else if(ch.getNodeName().equals("type"))
                {
                    catEntry.GetPronEntry().GetType().addElement(
                        getValue(ch));
                }
                else if(ch.getNodeName().equals("gender"))
                {
                    catEntry.GetPronEntry().SetGender(
                        getAttribute(ch, "type"));
                }
                else if(ch.getNodeName().equals("interrogative"))
                {
                    catEntry.GetPronEntry().SetInterrogative(true);
                }
            } 
        }
        return catEntry;
    }
    private static CatEntry getDetValues(Node node)
    {
        CatEntry catEntry = new CatEntry("det");
           // Visit all the child nodes
           for(Node ch = node.getFirstChild(); ch != null; ch=ch.getNextSibling())
        {
            // Recursively call if the child may have children
            if(ch instanceof Element) 
            {
                if(ch.getNodeName().equals("variants"))
                {
                    catEntry.GetDetEntry().SetVariants(getValue(ch));
                }
                else if(ch.getNodeName().equals("interrogative"))
                {
                    catEntry.GetDetEntry().SetInterrogative(true);
                }
                else if(ch.getNodeName().equals("demonstrative"))
                {
                    catEntry.GetDetEntry().SetDemonstrative(true);
                }
            } 
        }
        return catEntry;
    }
    private static CatEntry getAdvValues(Node node)
    {
        CatEntry catEntry = new CatEntry("adv");
           // Visit all the child nodes
           for(Node ch = node.getFirstChild(); ch != null; ch=ch.getNextSibling())
        {
            // Recursively call if the child may have children
            if(ch instanceof Element) 
            {
                if(ch.getNodeName().equals("variants"))
                {
                    catEntry.GetAdvEntry().GetVariants().addElement(
                        getValue(ch));
                }
                else if(ch.getNodeName().equals("modification"))
                {
                    catEntry.GetAdvEntry().GetModification().addElement(
                        getValue(ch));
                }
                else if(ch.getNodeName().equals("negative"))
                {
                    catEntry.GetAdvEntry().SetNegative(
                        getAttribute(ch, "type"));
                }
                else if(ch.getNodeName().equals("interrogative"))
                {
                    catEntry.GetAdvEntry().SetInterrogative(true);
                }
            } 
        }
        return catEntry;
    }
    private static CatEntry getAdjValues(Node node)
    {
        CatEntry catEntry = new CatEntry("adj");
           // Visit all the child nodes
           for(Node ch = node.getFirstChild(); ch != null; ch=ch.getNextSibling())
        {
            // Recursively call if the child may have children
            if(ch instanceof Element) 
            {
                if(ch.getNodeName().equals("variants"))
                {
                    catEntry.GetAdjEntry().GetVariants().addElement(
                        getValue(ch));
                }
                else if(ch.getNodeName().equals("position"))
                {
                    catEntry.GetAdjEntry().GetPosition().addElement(
                        getAttribute(ch, "type"));
                }
                else if(ch.getNodeName().equals("compl"))
                {
                    catEntry.GetAdjEntry().GetCompl().addElement(getValue(ch));
                }
                else if(ch.getNodeName().equals("nominalization"))
                {
                    catEntry.GetAdjEntry().GetNominalization().addElement(
                        getValue(ch));
                }
                else if(ch.getNodeName().equals("stative"))
                {
                    catEntry.GetAdjEntry().SetStative(true);
                }
            } 
        }
        return catEntry;
    }
    private static String getValue(Node node)
    {
        String value = null;
           // Visit all the child nodes
           for(Node ch = node.getFirstChild(); ch != null; ch=ch.getNextSibling())
        {
            // Recursively call if the child may have children
            if(ch instanceof Element || ch instanceof EntityReference) 
            {
                getValue(ch);
            } 
            else if(ch instanceof Text) 
            {
                value = ch.getNodeValue();    
            }
        }
        return value;
    }
    private static String getAttribute(Node node, String name)
    {
        String attribute = new String();
        NamedNodeMap nnm = node.getAttributes();
        if(nnm.getLength() == 1)
        {
            Node item = nnm.item(0);
            if(item.getNodeName().equals(name))
            {
                attribute = item.getNodeValue();
            }
        }
        return attribute;
    }
    private static Vector<String> getAttributes(Node node)
    {
        Vector<String> positions = new Vector<String>();
        NamedNodeMap nnm = node.getAttributes();
        for(int i = 0; i < nnm.getLength(); i++)
        {
            Node item = nnm.item(i);
            if(item.getNodeName().equals("type"))
            {
                positions.addElement(item.getNodeValue());
            }
        }
        return positions;
    }
    // check grammer for the read-in line
    private static void CheckLine(CheckSt st, CheckSt catSt,
        LineObject lineObject, Vector<LexRecord> lexRecords)
    {
        boolean flag = false;
        boolean printFlag = true;
        // check input for quiting
        flag = CheckGrammer.Check(lineObject, printFlag, st, catSt, false);
        // add lexRecord to output
        if((st.GetCurState() == CheckSt.CHECK_BASE)
        && (flag == true))
        {
            lexRecords.add(CheckGrammer.GetLexRecord());
        }
    }
    private static LexRecord CheckLine(CheckSt st, CheckSt catSt,
        LineObject lineObject)
    {
        boolean flag = false;
        boolean printFlag = true;
        // check input for quiting
        flag = CheckGrammer.Check(lineObject, printFlag, st, catSt, false);
        LexRecord lexRecord = new LexRecord();
        // add lexRecord to output
        if((st.GetCurState() == CheckSt.CHECK_BASE)
        && (flag == true))
        {
            lexRecord = CheckGrammer.GetLexRecord();
        }
        return lexRecord;
    }
    private static boolean CheckLine2(CheckSt st, CheckSt catSt,
        LineObject lineObject)
    {
        boolean flag = false;
        boolean printFlag = true;
        // check input for quiting
        flag = CheckGrammer.Check(lineObject, printFlag, st, catSt, false);
        if(flag == false)
        {
            return flag;
        }
        // update number of lexRecords
        if((st.GetCurState() == CheckSt.CHECK_BASE)
        && (flag == true))
        {
            lexRecNum_++;
        }
        return flag;
    }
    private static int CheckLine(CheckSt st, CheckSt catSt,
        LineObject lineObject, Vector<LexRecord> lexRecords, int recordNum)
    {
        boolean flag = false;
        boolean printFlag = true;
        // check input for quitiing
        flag = CheckGrammer.Check(lineObject, printFlag, st, catSt, false);
        // add lexRecord to output
        if((st.GetCurState() == CheckSt.CHECK_BASE)
        && (flag == true))
        {
            lexRecords.add(CheckGrammer.GetLexRecord());
            recordNum++;
        }
        return recordNum;
    }
    private static int lexRecNum_ = 0;
}
