package gov.nih.nlm.nls.lexCheck.Api;
import java.util.*;
import java.io.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Gram.*;
/*****************************************************************************
* This class provides an Java Api for converting lexical records in text format 
* to Xml output.  The input can be a file or a string.
*
* <p><b>History:</b>
* <ul>
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2013
****************************************************************************/
public class ToXmlApi
{
    // public method
    /**
    * A method to convert lexical records from a text file into Xml format.
    *
    * @param inFile  the file name of the lexical records in text format
    *
    * @return  an ApiOutput object includes total records number and Xml format
    *
    * @deprecated as in 2006, replaced by ToXmlFromTextFile(string)
    */
    @Deprecated public static ApiOutput ToXmlFromFile(String inFile) 
    {
        return ToXmlFromTextFile(inFile);
    }
    /**
    * A method to convert lexical records from a text file into Xml format.
    *
    * @param inFile  the file name of the lexical records in text format
    *
    * @return  an ApiOutput object includes total records number and Xml format
    */
    public static ApiOutput ToXmlFromTextFile(String inFile) 
    {
        CheckSt st = new CheckSt();     // upper level  Check state
        CheckSt catSt = new CheckSt(CheckSt.CHECK_CAT_ENTRY);
            // Category dependant Check state
        LineObject lineObject = new LineObject();
        StringBuffer xmlOut = new StringBuffer(
            "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
        xmlOut.append("<lexRecords>\n");
        int recordNum = 0;
        try
        {
            BufferedReader inReader = new BufferedReader(new InputStreamReader(
                new FileInputStream(inFile), "UTF-8"));
            while(lineObject != null)
            {
                // check if read in the next line
                if(lineObject.IsGoToNext() == true)
                {
                    lineObject.SetLine(inReader.readLine());
                    lineObject.IncreaseLineNum();
                }
                if(lineObject.GetLine() == null)
                {
                    break;
                }
                recordNum = CheckLine(st, catSt, lineObject, xmlOut, recordNum);
            }
            inReader.close();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        xmlOut.append("</lexRecords>\n");
        ApiOutput apiOutput = new ApiOutput(xmlOut.toString(), recordNum);
        return apiOutput;
    }
    /**
    * A method to convert lexical records in text format (string) into Xml 
    * format.  
    * The output of this method uses Unix Line separator as line separator.
    *
    * @param text  the lexical records in text format
    *
    * @return  an ApiOutput object includes total record number and Xml format
    */
    public static ApiOutput ToXmlFromText(String text) 
    {
        CheckSt st = new CheckSt();        // upper level  Check state
        CheckSt catSt = new CheckSt(CheckSt.CHECK_CAT_ENTRY);
            // Category dependant Check state
        LineObject lineObject = new LineObject();
        StringBuffer xmlOut = new StringBuffer(
            "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
        xmlOut.append("<lexRecords>\n");
        int recordNum = 0;
        // Convert to Unix line Separator for different platform
        String unixText = Convert.ToUnixLineSeparator(text);
        StringTokenizer buf = 
            new StringTokenizer(unixText, Convert.UNIX_LINE_SEP);
        while(lineObject != null)
        {
            // check if read in the next line
            if(lineObject.IsGoToNext() == true)
            {
                if(buf.hasMoreTokens() == false)
                {
                    break;
                }
                lineObject.SetLine(buf.nextToken());
                lineObject.IncreaseLineNum();
            }
            recordNum = CheckLine(st, catSt, lineObject, xmlOut, recordNum);
        }
        xmlOut.append("</lexRecords>\n");
        ApiOutput apiOutput = new ApiOutput(xmlOut.toString(), recordNum);
        return apiOutput;
    }
    /**
    * A method to generate inflection variables from a file 
    * contains lexical records in text format.
    * This method is the same as GetInflVarsFromTextFile(inFile).
    *
    * @param inFile  the file name of the lexical records in text format
    *
    * @return  a vector of elements of InflectionalVar
    *
    * @deprecated as in 2006, replaced by 
    * ToInflVarsApi.GetInflVarsFromTextFile(String) 
    */
    @Deprecated public static Vector<InflVar> GetInflVars(String inFile) 
    {
        return ToInflVarsApi.GetInflVarsFromTextFile(inFile);
    }
    /**
    * A unit test driver for ToXmlApi.
    */
    public static void main(String[] args)
    {
        String bat = 
"{base=bat\nentry=E0012112\n\tcat=noun\n\tvariants=reg\nsignature=vanni\n}";
        String molt =
"{base=molt\nspelling_variant=moult\nentry=E0040723\n\tcat=noun\n\tvariants=reg\nsignature=vanni\n}\n{base=molt\nspelling_variant=moult\nentry=E0040724\n\tcat=verb\n\tvariants=reg\n\tintran\n\ttran=np\n\tnominalization=molting|noun|E0412675\nsignature=vanni\n}";
        String inFile = "molt"; 
        // Go through LexCheck
        /***
        System.out.println(molt);
        System.out.print(ToXmlFromText(molt).GetXml());
        System.out.println("---- Record Num: " 
            + ToXmlFromText(molt).GetRecordNum());
        System.out.print(ToXmlFromText(bat).GetXml());
        System.out.println("---- Record Num: " 
            + ToXmlFromText(bat).GetRecordNum());
        ***/
        System.out.print(ToXmlFromFile(inFile).GetXml());
        System.out.println("---- Record Num: " 
            + ToXmlFromFile(inFile).GetRecordNum());
    }
    // check grammer for the read-in line 
    private static int CheckLine(CheckSt st, CheckSt catSt, 
        LineObject lineObject, StringBuffer xmlOut, int recordNum) 
    {
        boolean flag = false;
        boolean printFlag = true;
        // check input for quitiing
        flag = CheckGrammer.Check(lineObject, printFlag, st, catSt, false);
        // print out lex record 
        if((st.GetCurState() == CheckSt.CHECK_BASE)
        && (flag == true))
        {
            xmlOut.append(CheckGrammer.GetLexRecord().GetXml());
            recordNum++;
        }
        return recordNum;
    }
}
