package gov.nih.nlm.nls.lexCheck.Cat.Adv;
import java.util.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.CkLib.*;
/*****************************************************************************
* This class checks allowable filler for syntactical category of adverb.
*
* <p><b>History:</b>
* <ul>
* </ul>
*
* @author NLM LVG Development Team
*
* @version    V-2013
****************************************************************************/
public class CheckAdv
{
    // public method
    /**
    * Check a line to see if it is a legel filler.
    *
    * @param   lineObject  the java object of a input line
    * @param   printFlag   the flag for print out check result
    * @param   catSt       the state of category
    * @param   lexObj   the current object of lexical record
    * @param   debugFlag   the flag for print out debug information
    *
    * @return  true or false for legal and illegal filler.
    */
    public static boolean Check(LineObject lineObject, boolean printFlag,
        CheckSt catSt, LexRecord lexObj, boolean debugFlag)
    {
        boolean flag = true;
        int curSt = catSt.GetCurState();
        // reset the state if it is first time
        if(curSt == CheckSt.CHECK_CAT_ENTRY)
        {
            curSt = CheckSt.CHECK_ADV_VARIANTS;
            catSt.SetCurState(curSt);
        }
        switch(curSt)
        {
            case CheckSt.CHECK_ADV_VARIANTS:
                flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                    checkVariants_, new UpdateAdvVariants(), 
                    CheckCode.FQ_ONE_MANY, true);
                PrintStep(CheckSt.CHECK_ADV_VARIANTS, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_ADV_INTERROGATIVE:
                flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                    checkInterrogative_, new UpdateAdvInterrogative(), 
                    CheckCode.FQ_ZERO_ONE_WHOLE_LINE, false);
                PrintStep(CheckSt.CHECK_PRON_GENDER, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_ADV_MODIFICATION:
                flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                    checkModification_, new UpdateAdvModification(), 
                    CheckCode.FQ_ONE_MANY, true);
                PrintStep(CheckSt.CHECK_ADV_MODIFICATION, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_ADV_NEGATIVE:
                flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                    checkNegative_, new UpdateAdvNegative(), 
                    CheckCode.FQ_ZERO_ONE_WHOLE_LINE, false);
                // if no negative, check if there is broad_negative
                if(flag == false)
                {
                    catSt.SetCurState(CheckSt.CHECK_ADV_NEGATIVE);
                    catSt.SetLastState(CheckSt.CHECK_ADV_MODIFICATION);
                    flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                        checkBroadNegative_, new UpdateAdvNegative(), 
                        CheckCode.FQ_ZERO_ONE_WHOLE_LINE, false);
                }
                PrintStep(CheckSt.CHECK_ADV_NEGATIVE, debugFlag, 
                    lineObject.GetLine());
        }
        return flag;
    }
    // private method
    private static void PrintStep(int state, boolean debugFlag, String line)
    {
        if(debugFlag == true)
        {
            switch(state)
            {
                case CheckSt.CHECK_ADV_VARIANTS:
                    System.out.println("-- Checked Adv Variants: '" + line 
                        + "'");
                    break;
                case CheckSt.CHECK_ADV_INTERROGATIVE:
                    System.out.println("-- Checked Adv Interrogative: '" + line 
                        + "'");
                    break;
                case CheckSt.CHECK_ADV_MODIFICATION:
                    System.out.println("-- Checked Adv Modification: '" 
                        + line + "'");
                    break;
                case CheckSt.CHECK_ADV_NEGATIVE:
                    System.out.println("-- Checked Adv Negative: '" + line 
                        + "'");
                    break;
            }
        }
    }
    // data member
    private static CheckObject checkVariants_ = null;
    private static HashSet<String> variantsNextStartStrs_ 
        = new HashSet<String>(9);
    private static CheckObject checkInterrogative_ = null;
    private static HashSet<String> interrogativeNextStartStrs_ 
        = new HashSet<String>(8);
    private static CheckObject checkModification_ = null;
    private static HashSet<String> modificationNextStartStrs_ 
        = new HashSet<String>(7);
    private static CheckObject checkNegative_ = null;
    private static CheckObject checkBroadNegative_ = null;
    private static HashSet<String> negativeNextStartStrs_ 
        = new HashSet<String>(6);
    static
    {
        // slot variants=
        variantsNextStartStrs_.add("\tinterrogative");
        variantsNextStartStrs_.add("\tmodification_type=");
        variantsNextStartStrs_.add("\tnegative");
        variantsNextStartStrs_.add("\tbroad_negative");
        variantsNextStartStrs_.add("\tacronym_of=");
        variantsNextStartStrs_.add("\tabbreviation_of=");
        variantsNextStartStrs_.add("annotation=");
        variantsNextStartStrs_.add("signature=");
        variantsNextStartStrs_.add("}");
        checkVariants_ = new CheckObject("\tvariants=", 
            ErrMsg.ADV_VAR_SLOT_ERR, ErrMsg.ADV_VAR_FILLER_ERR, 
            CheckSt.CHECK_ADV_INTERROGATIVE, variantsNextStartStrs_,
            new CheckFormatAdvVariants());
        // slot interrogative
        interrogativeNextStartStrs_.add("\tmodification_type=");
        interrogativeNextStartStrs_.add("\tnegative");
        interrogativeNextStartStrs_.add("\tbroad_negative");
        interrogativeNextStartStrs_.add("\tacronym_of=");
        interrogativeNextStartStrs_.add("\tabbreviation_of=");
        interrogativeNextStartStrs_.add("annotation=");
        interrogativeNextStartStrs_.add("signature=");
        interrogativeNextStartStrs_.add("}");
        checkInterrogative_ = new CheckObject("\tinterrogative", 
            ErrMsg.ADV_INTERROGATIVE_SLOT_ERR, ErrMsg.NONE,
            CheckSt.CHECK_ADV_MODIFICATION, interrogativeNextStartStrs_, null);
        // slot modification_type=
        modificationNextStartStrs_.add("\tnegative");
        modificationNextStartStrs_.add("\tbroad_negative");
        modificationNextStartStrs_.add("\tacronym_of=");
        modificationNextStartStrs_.add("\tabbreviation_of=");
        modificationNextStartStrs_.add("annotation=");
        modificationNextStartStrs_.add("signature=");
        modificationNextStartStrs_.add("}");
        checkModification_ = new CheckObject("\tmodification_type=", 
            ErrMsg.ADV_MODIFICATION_SLOT_ERR, 
            ErrMsg.ADV_MODIFICATION_FILLER_ERR,
            CheckSt.CHECK_ADV_NEGATIVE, modificationNextStartStrs_, 
            new CheckFormatAdvModification());
        // slot negative
        negativeNextStartStrs_.add("\tacronym_of=");
        negativeNextStartStrs_.add("\tabbreviation_of=");
        negativeNextStartStrs_.add("annotation=");
        negativeNextStartStrs_.add("signature=");
        negativeNextStartStrs_.add("}");
        checkNegative_ = new CheckObject("\tnegative", 
            ErrMsg.NONE, ErrMsg.NONE,
            CheckSt.CHECK_ACRONYM, negativeNextStartStrs_, null);
        checkBroadNegative_ = new CheckObject("\tbroad_negative", 
            ErrMsg.ADV_BROAD_NEGATIVE_SLOT_ERR, ErrMsg.NONE,
            CheckSt.CHECK_ACRONYM, negativeNextStartStrs_, null);
    }
}
