package gov.nih.nlm.nls.lexCheck.Cat;
import java.util.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Cat.Auxi.*;
import gov.nih.nlm.nls.lexCheck.Cat.Modal.*;
import gov.nih.nlm.nls.lexCheck.Cat.Adj.*;
import gov.nih.nlm.nls.lexCheck.Cat.Adv.*;
import gov.nih.nlm.nls.lexCheck.Cat.Verb.*;
import gov.nih.nlm.nls.lexCheck.Cat.Noun.*;
import gov.nih.nlm.nls.lexCheck.Cat.Pron.*;
import gov.nih.nlm.nls.lexCheck.Cat.Det.*;
/*****************************************************************************
* This class checks variants, compelments,  nd otehr attributes of an entry 
* based on category.
*
* <p><b>History:</b>
*
* @author NLM Lexical System Development Team
*
* @version    V-2013
****************************************************************************/
public class CheckCatEntry
{
    // public method
    /**
    * Check the code of the input line.
    *
    * @param   lineObject  the java object of the input line
    * @param   printFlag   the flag for print out check result
    * @param   st          the state of current checking
    * @param   catSt       the state of current category
    * @param   lexObj      the current object of lexical record
    * @param   nextState    the next state after this check
    * @param   debugFlag   the boolean flag of printing out debug information
    *
    * @return  true or false for a legal or illegal code of a line
    */
    public static boolean Check(LineObject lineObject, boolean printFlag,
        CheckSt st, CheckSt catSt, LexRecord lexObj, int nextState, 
        boolean debugFlag)
    {
        boolean flag = false;
        String category = lexObj.GetCategory();
        if(category.equals(CheckFormatCat.VERB) == true)
        {
            flag = CheckVerb.Check(lineObject, printFlag, catSt, lexObj,
                debugFlag);
        }
        else if(category.equals(CheckFormatCat.AUX) == true)
        {
            flag = CheckAux.Check(lineObject, printFlag, catSt, lexObj, 
                debugFlag);
        }
        else if(category.equals(CheckFormatCat.MODAL) == true)
        {
            flag = CheckModal.Check(lineObject, printFlag, catSt, lexObj, 
                debugFlag);
        }
        else if(category.equals(CheckFormatCat.NOUN) == true)
        {
            flag = CheckNoun.Check(lineObject, printFlag, catSt, lexObj,
                debugFlag);
        }
        else if(category.equals(CheckFormatCat.PRON) == true)
        {
            flag = CheckPron.Check(lineObject, printFlag, catSt, lexObj, 
                debugFlag);
        }
        else if(category.equals(CheckFormatCat.ADJ) == true)
        {
            flag = CheckAdj.Check(lineObject, printFlag, catSt, lexObj, 
                debugFlag);
        }
        else if(category.equals(CheckFormatCat.ADV) == true)
        {
            flag = CheckAdv.Check(lineObject, printFlag, catSt, lexObj, 
                debugFlag);
        }
        else if(category.equals(CheckFormatCat.PREP) == true)
        {
            flag = CheckNone(lineObject, catSt, nextState);
        }
        else if(category.equals(CheckFormatCat.CONJ) == true)
        {
            flag = CheckNone(lineObject, catSt, nextState);
        }
        else if(category.equals(CheckFormatCat.COMPL) == true)
        {
            flag = CheckNone(lineObject, catSt, nextState);
        }
        else if(category.equals(CheckFormatCat.DET) == true)
        {
            flag = CheckDet.Check(lineObject, printFlag, catSt, lexObj, 
                debugFlag);
        }
        // set state to nextState to leave cat entry
        if(catSt.GetCurState() == nextState)
        {
            st.SetCurState(nextState);
            catSt.SetCurState(CheckSt.CHECK_CAT_ENTRY);
        }
        return flag;
    }
    // private method
    // No category dependent entry to be checked
    private static boolean CheckNone(LineObject lineObject, CheckSt catSt,
        int nextState)
    {
        lineObject.SetGoToNext(false);
        catSt.UpdateCurState(nextState);
        return true;
    }
}
