package gov.nih.nlm.nls.lexCheck.Cat.Verb;
import java.util.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Compl.*;
/*****************************************************************************
* This class checks format of ditransitive complements for verb.
*
* <p><b>History:</b>
* <ul>
* </ul>
*
* @author NLM LVG Development Team
*
* @version    V-2013
****************************************************************************/
public class CheckFormatVerbDitran implements CheckFormat
{
    // public method
    /**
    * Check a line to see if it is a legal format of ditransitive 
    * complements of verb.
    *
    * @param   inCode   the filler of verb ditransitive complements.
    *
    * @return  true or false for legal and illegal filler.
    */
    public boolean IsLegalFormat(String inCode)
    {
        boolean flag = false;
        int partIndex = inCode.indexOf(";part(");
        int passiveIndex = inCode.indexOf(";nopass");
        int datmvtIndex = inCode.indexOf(";datmvt");
        int noPrtmvtIndex = inCode.indexOf(";noprtmvt");
        String filler = null;
        String particle = null;
        boolean passive = false;
        boolean noPrtmvt = false;
        // get filler
        if(partIndex != -1)
        {
            filler = inCode.substring(0, partIndex);
            // Get particle code
            if(passiveIndex != -1)
            {
                particle = inCode.substring(partIndex, passiveIndex);
            }
            else if(datmvtIndex != -1)
            {
                particle = inCode.substring(partIndex, datmvtIndex);
            }
            else if(noPrtmvtIndex != -1)
            {
                particle = inCode.substring(partIndex, noPrtmvtIndex);
            }
            else
            {
                particle = inCode.substring(partIndex);
            }
        }
        else
        {
            if(passiveIndex != -1)
            {
                filler = inCode.substring(0, passiveIndex);
            }
            else if(datmvtIndex != -1)
            {
                filler = inCode.substring(0, datmvtIndex);
            }
            else if(noPrtmvtIndex != -1)
            {
                filler = inCode.substring(0, noPrtmvtIndex);
            }
            else
            {
                filler = inCode;
            }
        }
        // check filler & particle code
        if(particle != null)
        {
            flag = CheckDitranFiller(filler) && CheckParticle.IsLegal(particle);
        }
        else
        {
            flag = CheckDitranFiller(filler);
        }
        return flag;
    }
    // private method
    private boolean CheckDitranFiller(String filler)
    {
        boolean flag = false;
        String filler1 = new String();
        String filler2 = new String();
        if(filler.startsWith("pphr(") == true)
        {
            int index = filler.indexOf("),");
            filler1 = filler.substring(0, index+1);
            filler2 = filler.substring(index+2);
        }
        else
        {
            int index = filler.indexOf(",");
            filler1 = filler.substring(0, index);
            filler2 = filler.substring(index+1);
        }
        
        flag = CheckFiller1(filler1) && CheckFiller2(filler2);
        
        return flag;
    }
    private boolean CheckFiller1(String filler)
    {
        boolean flag = false;
        if(filler1_.contains(filler) == true)
        {
            flag = true;
        }
        else if(filler.startsWith("fincomp(") == true)
        {
            flag = CheckFinComp.IsLegal(filler);
        }
        else if(filler.startsWith("ingcomp:") == true)
        {
            flag = CheckIngComp.IsLegal(filler);
        }
        else if(filler.startsWith("np|") == true)
        {
            flag = CheckNpComp.IsLegal(filler);
        }
        else if(filler.startsWith("pphr(") == true)
        {
            flag = CheckPphr.IsLegal(filler);
        }
        return flag;
    }
    private boolean CheckFiller2(String filler)
    {
        boolean flag = false;
        if(filler2_.contains(filler) == true)
        {
            flag = true;
        }
        else if(filler.startsWith("fincomp(") == true)
        {
            flag = CheckFinComp.IsLegal(filler);
        }
        else if(filler.startsWith("np|") == true)
        {
            flag = CheckNpComp.IsLegal(filler);
        }
        else if(filler.startsWith("pphr(") == true)
        {
            flag = CheckPphr.IsLegal(filler);
        }
        else if(filler.startsWith("whfincomp:") == true)
        {
            flag = CheckWhfinComp.IsLegal(filler);
        }
        else if(filler.startsWith("whinfcomp:") == true)
        {
            flag = CheckWhinfComp.IsLegal(filler);
        }
        return flag;
    }
    // data member
    private final static int LEGAL_FILLER1_NUM = 1;
    private static HashSet<String> filler1_ 
        = new HashSet<String>(LEGAL_FILLER1_NUM);
    private final static int LEGAL_FILLER2_NUM = 2;
    private static HashSet<String> filler2_ 
        = new HashSet<String>(LEGAL_FILLER2_NUM);
    static
    {
        filler1_.add("np");
        filler2_.add("np");
        filler2_.add("whfincomp");
    }
}
