package gov.nih.nlm.nls.lexCheck.Cat.Verb;
import java.util.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Compl.*;
/*****************************************************************************
* This class checks format of linking complements for verb.
*
* <p><b>History:</b>
* <ul>
* </ul>
*
* @author NLM LVG Development Team
*
* @version    V-2013
****************************************************************************/
public class CheckFormatVerbLink implements CheckFormat
{
    // public method
    /**
    * Check a line to see if it is a legal format of linking 
    * complements of verb.
    *
    * @param   inCode   the filler of verb linking complements.
    *
    * @return  true or false for legal and illegal filler.
    */
    public boolean IsLegalFormat(String inCode)
    {
        boolean flag = false;
        int partIndex = inCode.indexOf(";part(");
        String filler = null;
        String particle = null;
        // get filler
        if(partIndex != -1)
        {
            filler = inCode.substring(0, partIndex);
            particle = inCode.substring(partIndex);
        }
        else
        {
            filler = inCode;
        }
        // check filler & particle code
        if(particle != null)
        {
            flag = CheckLinkFiller(filler) && CheckParticle.IsLegal(particle);
        }
        else
        {
            flag = CheckLinkFiller(filler);
        }
        return flag;
    }
    // private method
    private boolean CheckLinkFiller(String filler)
    {
        boolean flag = false;
        if(filler_.contains(filler) == true)
        {
            flag = true;
        }
        else if(filler.startsWith("edcomp:") == true)
        {
            flag = CheckEdComp.IsLegal(filler);
        }
        else if(filler.startsWith("fincomp(") == true)
        {
            flag = CheckFinComp.IsLegal(filler);
        }
        else if(filler.startsWith("infcomp:") == true)
        {
            flag = CheckInfComp.IsLegal(filler);
        }
        else if(filler.startsWith("ingcomp:") == true)
        {
            flag = CheckIngComp.IsLegal(filler);
        }
        else if(filler.startsWith("pphr(") == true)
        {
            flag = CheckPphr.IsLegal(filler);
        }
        return flag;
    }
    // data member
    private final static int LEGAL_FILLER_NUM = 3;
    private static HashSet<String> filler_ 
        = new HashSet<String>(LEGAL_FILLER_NUM);
    static
    {
        filler_.add("adj");
        filler_.add("advbl");
        filler_.add("np");
    }
}
