package gov.nih.nlm.nls.lexCheck.Cat.Verb;
import java.util.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.CkLib.*;
/*****************************************************************************
* This class checks filler for verb.
*
* <p><b>History:</b>
* <ul>
* </ul>
*
* @author NLM LVG Development Team
*
* @version    V-2013
****************************************************************************/
public class CheckVerb
{
    // public method
    /**
    * Check a line to see if it is a legel filler.
    *
    * @param   lineObject  the java object of the input line
    * @param   printFlag   the flag for print out check result
    * @param   catSt       the state of category
    * @param   lexObj   the current object of lexical record
    * @param   debugFlag   the flag for print out debug information
    *
    * @return  true or false for legal and illegal filler.
    */
    public static boolean Check(LineObject lineObject, boolean printFlag,
        CheckSt catSt, LexRecord lexObj, boolean debugFlag)
    {
        boolean flag = true;
        int curSt = catSt.GetCurState();
        // reset the state if it is first time
        if(curSt == CheckSt.CHECK_CAT_ENTRY)
        {
            curSt = CheckSt.CHECK_VERB_VARIANTS;
            catSt.SetCurState(curSt);
        }
        // TBD: intran + tran + Ditran + link + cplxtran: one ~ many
        switch(curSt)
        {
            case CheckSt.CHECK_VERB_VARIANTS:
                flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                    checkVariants_, new UpdateVerbVariants(), 
                    CheckCode.FQ_ONE_MANY, true);
                PrintStep(CheckSt.CHECK_VERB_VARIANTS, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_VERB_INTRAN:
                flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                    checkIntran_, new UpdateVerbIntran(), 
                    CheckCode.FQ_ZERO_ONE_WHOLE_LINE, false);
                PrintStep(CheckSt.CHECK_VERB_INTRAN, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_VERB_INTRAN2:
                flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                    checkIntran2_, new UpdateVerbIntran(), 
                    CheckCode.FQ_ZERO_MANY, false);
                PrintStep(CheckSt.CHECK_VERB_INTRAN2, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_VERB_TRAN:
                flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                    checkTran_, new UpdateVerbTran(), 
                    CheckCode.FQ_ZERO_MANY, true);
                PrintStep(CheckSt.CHECK_VERB_TRAN, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_VERB_DITRAN:
                flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                    checkDitran_, new UpdateVerbDitran(), 
                    CheckCode.FQ_ZERO_MANY, true);
                PrintStep(CheckSt.CHECK_VERB_DITRAN, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_VERB_LINK:
                flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                    checkLink_, new UpdateVerbLink(), 
                    CheckCode.FQ_ZERO_MANY, true);
                PrintStep(CheckSt.CHECK_VERB_LINK, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_VERB_CPLXTRAN:
                flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                    checkCplxtran_, new UpdateVerbCplxtran(), 
                    CheckCode.FQ_ZERO_MANY, true);
                PrintStep(CheckSt.CHECK_VERB_CPLXTRAN, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_VERB_NOMINALIZATION:
                if(CheckVerbComplNum(lexObj) == false)
                {
                    String line = lineObject.GetLine();
                    ErrMsg.PrintErrMsg(printFlag, ErrMsg.VERB_COMPL_NUM_ERR,
                        lineObject, line, 0, line.length(), true);
                    flag = false;
                    break;
                }
                if(flag == true)
                {
                    flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                        checkNominalization_, new UpdateVerbNominalization(), 
                        CheckCode.FQ_ZERO_MANY, true);
                }
                PrintStep(CheckSt.CHECK_VERB_NOMINALIZATION, debugFlag, 
                    lineObject.GetLine());
                break;
        }
        return flag;
    }
    // private method
    private static boolean CheckVerbComplNum(LexRecord lexObj)
    {
        boolean flag = false;
        VerbEntry verbEntry = lexObj.GetCatEntry().GetVerbEntry();
        if((verbEntry.GetIntran().size() > 0)
        || (verbEntry.GetTran().size() > 0)
        || (verbEntry.GetDitran().size() > 0)
        || (verbEntry.GetLink().size() > 0)
        || (verbEntry.GetCplxtran().size() > 0))
        {
            flag = true;
        }
        return flag;
    }
    private static void PrintStep(int state, boolean debugFlag, String line)
    {
        if(debugFlag == true)
        {
            switch(state)
            {
                case CheckSt.CHECK_VERB_VARIANTS:
                    System.out.println("-- Checked Verb Variants: '" + line 
                        + "'");
                    break;
                case CheckSt.CHECK_VERB_INTRAN:
                    System.out.println("-- Checked Verb Intransitive: '" 
                        + line + "'");
                    break;
                case CheckSt.CHECK_VERB_INTRAN2:
                    System.out.println("-- Checked Verb Intransitive 2: '" 
                        + line + "'");
                    break;
                case CheckSt.CHECK_VERB_TRAN:
                    System.out.println("-- Checked Verb Transitive: '" 
                        + line + "'");
                    break;
                case CheckSt.CHECK_VERB_DITRAN:
                    System.out.println("-- Checked Verb Ditransitive: '" 
                        + line + "'");
                    break;
                case CheckSt.CHECK_VERB_LINK:
                    System.out.println("-- Checked Verb Linking: '" 
                        + line + "'");
                    break;
                case CheckSt.CHECK_VERB_CPLXTRAN:
                    System.out.println("-- Checked Verb Complex Transitive: '" 
                        + line + "'");
                    break;
                case CheckSt.CHECK_VERB_NOMINALIZATION:
                    System.out.println("-- Checked Verb Nominalization: '" 
                        + line + "'");
                    break;
            }
        }
    }
    // data member
    private static CheckObject checkVariants_ = null;
    private static HashSet<String> variantsNextStartStrs_ 
        = new HashSet<String>(11);
    private static CheckObject checkIntran_ = null;
    private static HashSet<String> intranNextStartStrs_ 
        = new HashSet<String>(11);
    private static CheckObject checkIntran2_ = null;
    private static HashSet<String> intran2NextStartStrs_ 
        = new HashSet<String>(10);
    private static CheckObject checkTran_ = null;
    private static HashSet<String> tranNextStartStrs_ 
        = new HashSet<String>(9);
    private static CheckObject checkDitran_ = null;
    private static HashSet<String> ditranNextStartStrs_ 
        = new HashSet<String>(8);
    private static CheckObject checkLink_ = null;
    private static HashSet<String> linkNextStartStrs_ 
        = new HashSet<String>(7);
    private static CheckObject checkCplxtran_ = null;
    private static HashSet<String> cplxtranNextStartStrs_ 
        = new HashSet<String>(6);
    private static CheckObject checkNominalization_ = null;
    private static HashSet<String> nominalizationNextStartStrs_ 
        = new HashSet<String>(5);
    static
    {
        // slot variants=
        variantsNextStartStrs_.add("\tintran");
        variantsNextStartStrs_.add("\ttran=");
        variantsNextStartStrs_.add("\tditran=");
        variantsNextStartStrs_.add("\tlink=");
        variantsNextStartStrs_.add("\tcplxtran=");
        variantsNextStartStrs_.add("\tnominalization=");
        variantsNextStartStrs_.add("\tacronym_of=");
        variantsNextStartStrs_.add("\tabbreviation_of=");
        variantsNextStartStrs_.add("annotation=");
        variantsNextStartStrs_.add("signature=");
        variantsNextStartStrs_.add("}");
        checkVariants_ = new CheckObject("\tvariants=", 
            ErrMsg.VERB_VAR_SLOT_ERR, ErrMsg.VERB_VAR_FILLER_ERR, 
            CheckSt.CHECK_VERB_INTRAN, variantsNextStartStrs_,
            new CheckFormatVerbVariants());
        // slot intran
        intranNextStartStrs_.add("\tintran;part(");
        intranNextStartStrs_.add("\ttran=");
        intranNextStartStrs_.add("\tditran=");
        intranNextStartStrs_.add("\tlink=");
        intranNextStartStrs_.add("\tcplxtran=");
        intranNextStartStrs_.add("\tnominalization=");
        intranNextStartStrs_.add("\tacronym_of=");
        intranNextStartStrs_.add("\tabbreviation_of=");
        intranNextStartStrs_.add("annotation=");
        intranNextStartStrs_.add("signature=");
        intranNextStartStrs_.add("}");
        checkIntran_ = new CheckObject("\tintran", 
            ErrMsg.VERB_INTRAN_SLOT_ERR, ErrMsg.VERB_INTRAN_FILLER_ERR,
            CheckSt.CHECK_VERB_INTRAN2, intranNextStartStrs_, 
            null);
        intran2NextStartStrs_.add("\ttran=");
        intran2NextStartStrs_.add("\tditran=");
        intran2NextStartStrs_.add("\tlink=");
        intran2NextStartStrs_.add("\tcplxtran=");
        intran2NextStartStrs_.add("\tnominalization=");
        intran2NextStartStrs_.add("\tacronym_of=");
        intran2NextStartStrs_.add("\tabbreviation_of=");
        intran2NextStartStrs_.add("annotation=");
        intran2NextStartStrs_.add("signature=");
        intran2NextStartStrs_.add("}");
        checkIntran2_ = new CheckObject("\tintran;part(", 
            ErrMsg.VERB_INTRAN2_SLOT_ERR, ErrMsg.VERB_INTRAN2_FILLER_ERR,
            CheckSt.CHECK_VERB_TRAN, intran2NextStartStrs_, 
            new CheckFormatVerbIntran(), "(");
        // slot tran=
        tranNextStartStrs_.add("\tditran=");
        tranNextStartStrs_.add("\tlink=");
        tranNextStartStrs_.add("\tcplxtran=");
        tranNextStartStrs_.add("\tnominalization=");
        tranNextStartStrs_.add("\tacronym_of=");
        tranNextStartStrs_.add("\tabbreviation_of=");
        tranNextStartStrs_.add("annotation=");
        tranNextStartStrs_.add("signature=");
        tranNextStartStrs_.add("}");
        checkTran_ = new CheckObject("\ttran=", 
            ErrMsg.VERB_TRAN_SLOT_ERR, ErrMsg.VERB_TRAN_FILLER_ERR,
            CheckSt.CHECK_VERB_DITRAN, tranNextStartStrs_, 
            new CheckFormatVerbTran());
        // slot ditran=
        ditranNextStartStrs_.add("\tlink=");
        ditranNextStartStrs_.add("\tcplxtran=");
        ditranNextStartStrs_.add("\tnominalization=");
        ditranNextStartStrs_.add("\tacronym_of=");
        ditranNextStartStrs_.add("\tabbreviation_of=");
        ditranNextStartStrs_.add("annotation=");
        ditranNextStartStrs_.add("signature=");
        ditranNextStartStrs_.add("}");
        checkDitran_ = new CheckObject("\tditran=", 
            ErrMsg.VERB_DITRAN_SLOT_ERR, ErrMsg.VERB_DITRAN_FILLER_ERR,
            CheckSt.CHECK_VERB_LINK, ditranNextStartStrs_, 
            new CheckFormatVerbDitran());
        // slot link=
        linkNextStartStrs_.add("\tcplxtran=");
        linkNextStartStrs_.add("\tnominalization=");
        linkNextStartStrs_.add("\tacronym_of=");
        linkNextStartStrs_.add("\tabbreviation_of=");
        linkNextStartStrs_.add("annotation=");
        linkNextStartStrs_.add("signature=");
        linkNextStartStrs_.add("}");
        checkLink_ = new CheckObject("\tlink=", 
            ErrMsg.VERB_LINK_SLOT_ERR, ErrMsg.VERB_LINK_FILLER_ERR,
            CheckSt.CHECK_VERB_CPLXTRAN, linkNextStartStrs_, 
            new CheckFormatVerbLink());
        // slot cplxtran=
        cplxtranNextStartStrs_.add("\tnominalization=");
        cplxtranNextStartStrs_.add("\tacronym_of=");
        cplxtranNextStartStrs_.add("\tabbreviation_of=");
        cplxtranNextStartStrs_.add("annotation=");
        cplxtranNextStartStrs_.add("signature=");
        cplxtranNextStartStrs_.add("}");
        checkCplxtran_ = new CheckObject("\tcplxtran=", 
            ErrMsg.VERB_CPLXTRAN_SLOT_ERR, ErrMsg.VERB_CPLXTRAN_FILLER_ERR,
            CheckSt.CHECK_VERB_NOMINALIZATION, cplxtranNextStartStrs_, 
            new CheckFormatVerbCplxtran());
        // slot nominalization=
        nominalizationNextStartStrs_.add("\tacronym_of=");
        nominalizationNextStartStrs_.add("\tabbreviation_of=");
        nominalizationNextStartStrs_.add("annotation=");
        nominalizationNextStartStrs_.add("signature=");
        nominalizationNextStartStrs_.add("}");
        checkNominalization_ = new CheckObject("\tnominalization=", 
            ErrMsg.VERB_NOMINALIZATION_SLOT_ERR, 
            ErrMsg.VERB_NOMINALIZATION_FILLER_ERR, 
            CheckSt.CHECK_ACRONYM, nominalizationNextStartStrs_, 
            new CheckFormatVerbNominalization());
    }
}
