package gov.nih.nlm.nls.lexCheck.CheckCont;
import java.util.*;
import java.io.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Api.*;
/*****************************************************************************
* This class provides a Java class to check cross-ref EUI on abbreviation.
*
* <p><b>History:</b>
* <ul>
* <li>SCR-40, chlu, 04/23/2013, Add new features to check entire lexicon
* <li>SCR-49, chlu, 07/13/2015, add notBaseForm to exclude expansion
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2013
****************************************************************************/
public class CrossCheckAbbEui
{
    // private constructor
    private CrossCheckAbbEui()
    {
    }

	// lexRecord is automatically fixed
    public static boolean Check(LexRecord lexRecord,
		HashSet<String> notBaseFormSet) 
	{
		boolean validFlag = true;
		Vector<String> abbList = lexRecord.GetAbbreviations();

		String abbCat = lexRecord.GetCategory();
		for(int i = 0; i < abbList.size(); i++)
		{
			String abb = abbList.elementAt(i);
			// tokenize abbreviations
			int index1 = abb.indexOf(LexRecordUtil.FS_STR);
			String abbCit = new String();
			String abbEui  = new String();

			if(index1 > 0) // for abb = citation|Eui
			{
				abbCit = abb.substring(0, index1);
				abbEui = abb.substring(index1+1);
			}
			else	// for abb = citation
			{
				abbCit = abb;
			}

			// check cross-ref EUIs
			String citCat = abbCit + LexRecordUtil.FS_STR + abbCat;
			HashSet<String> euisByCitCat = 
				CrossCheckDupLexRecords.GetEuisByCitCat(citCat);
			
			if(euisByCitCat == null)	// eui does not exist
			{
				if(abbEui.length() > 0)	// eui exist
				{
					// auto-fix: remove EUI
					abbList.set(i, abbCit);

					// sent auto-fix msg
					validFlag = false;
					ErrMsgUtilLexicon.AddContentErrMsg(
						LexRecordUtil.CROSS_CHK_ABB,
						ErrMsgUtilLexicon.ERR_NO_EUI, 
						abb + " - None", lexRecord);
				}
				else	// no abbEui	
				{
					// no EUI matches: send msg to add to to-do list
					validFlag = false;
					// send errMsg only if expansion is not one of notBaseForm
					if(notBaseFormSet.contains(citCat) == false)
					{
						ErrMsgUtilLexicon.AddContentErrMsg(
							LexRecordUtil.CROSS_CHK_ABB,
							ErrMsgUtilLexicon.ERR_NEW_EUI, 
							abb + " - New", lexRecord);
					}
				}
			}
			else if (euisByCitCat.size() == 1)	// single EUI
			{
				Vector<String> euiList = new Vector<String>(euisByCitCat);
				String newEui = euiList.elementAt(0);
				if(abbEui.length() > 0)
				{
					if(euisByCitCat.contains(abbEui) == true)
					{
						// One EUI match, do nothing, correct
					}
					else
					{
						// sent warning msg
						validFlag = false;
						ErrMsgUtilLexicon.AddContentErrMsg(
							LexRecordUtil.CROSS_CHK_ABB,
							ErrMsgUtilLexicon.ERR_WRONG_EUI, 
							abb + " - " + newEui, lexRecord);
					}
				}
				else	// no abbEui => Add it in
				{
					// auto-fix
					String newAbb = abb + LexRecordUtil.FS_STR + newEui;
					abbList.set(i, newAbb);

					// sent out auto fix msg
					validFlag = false;
					ErrMsgUtilLexicon.AddContentErrMsg(
						LexRecordUtil.CROSS_CHK_ABB,
						ErrMsgUtilLexicon.ERR_MISS_EUI, 
						abb + " - " + newEui, lexRecord);
				}
			}
			else	// multiple EUIs
			{
				Vector<String> euiList = new Vector<String>(euisByCitCat);
				if(abbEui.length() > 0)
				{
					if(euisByCitCat.contains(abbEui) == true)
					{
						// One EUI match, do nothing, correct
					}
					else	// no eui even multiple matched EUI found
					{
						// sent warning msg
						validFlag = false;
						ErrMsgUtilLexicon.AddContentErrMsg(
							LexRecordUtil.CROSS_CHK_ABB,
							ErrMsgUtilLexicon.ERR_WRONG_EUIS, 
							abb + " - " + euiList, lexRecord);
					}
				}
				else	// no abbEui
				{
					// sent out msg
					validFlag = false;
					ErrMsgUtilLexicon.AddContentErrMsg(
						LexRecordUtil.CROSS_CHK_ABB,
						ErrMsgUtilLexicon.ERR_MISS_EUIS, 
						abb + " - " + euiList, lexRecord);
				}
			}
		}

		return validFlag;
	}

    // data member
}
