package gov.nih.nlm.nls.lexCheck.CheckCont;
import java.util.*;
import gov.nih.nlm.nls.lvg.Util.UnicodeUtil;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Api.*;
/*****************************************************************************
* This class provides a Java utility class of error message of content check.
* It is similar to the Lib.ErrMsg class
*
* <p><b>History:</b>
* <ul>
* <li>SCR-39, chlu, 03/28/2013, Add new features to check content
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2013
****************************************************************************/
public class ErrMsgUtil
{
	/**
	* Reset the error message.
	*/
    public static void ResetErrMsg()
    {
        errMsg_ = new String();
    }
	/**
	* Set the error message.
	* 
	* @param	errMsg	error message to be set
	*/
    public static void SetErrMsg(String errMsg)
    {
        errMsg_ = errMsg;
    }
	/**
	* Append the error message.
	* 
	* @param	errMsg	error message to be append
	*/
    public static void AppendErrMsg(String errMsg)
    {
        errMsg_ += errMsg;
    }

	/**
	* Add Content error message.
	*
	* @param	contentType	type of the checking content
	* @param	errType	found error type 
	* @param	msg	error message 
	* @param	errTypeStrs string of error type	
	* @param	errTypeNos no. of error type
	* @param	corssRefFlag flag for cross referenece	
	*/
    public static void AddContentErrMsg(int contentType, int errType, 
		String msg, String[][] errTypeStrs, int[] errTypeNos, 
		boolean corssRefFlag)
    {
		String contentTypeStr = LexRecordUtil.GetContentTypeStr(contentType);
		if(corssRefFlag == true)
		{
			contentTypeStr = LexRecordUtil.GetCrossRefTypeStr(contentType);
		}
		String errTypeStr = GetErrTypeStr(errType, errTypeStrs);
		String errFixStr = GetErrFixStr(errType, errTypeStrs);

		// update err msg
		String errMsg = "** Content Err in " + contentTypeStr + " - "
			+ errTypeStr + " @ [" + msg + "]" 
			+ errFixStr + GlobalVars.LS_STR;
        errMsg_ += errMsg;

		// update err no
		AddErrTypeNo(errType, errTypeNos);
    }
	/**
	* Add Content error message.
	*
	* @param	contentType		type of the checking content
	* @param	errType		found error type 
	* @param	curItem		where the error is
	* @param	lexRecord	the lexRecord is checked
	* @param	errTypeStrs 	string of error type	
	* @param	errTypeNos 		no. of error type
	* @param	corssRefFlag 	flag for cross referenece	
	*/
    public static void AddContentErrMsg(int contentType, int errType, 
		String curItem, LexRecord lexRecord, String[][] errTypeStrs,
		int[] errTypeNos, boolean corssRefFlag)
    {
		String contentTypeStr = LexRecordUtil.GetContentTypeStr(contentType);
		if(corssRefFlag == true)
		{
			contentTypeStr = LexRecordUtil.GetCrossRefTypeStr(contentType);
		}
		String errTypeStr = GetErrTypeStr(errType, errTypeStrs);
		String errFixStr = GetErrFixStr(errType, errTypeStrs);

		// update err msg
		String errMsg = "** Content Err in " + contentTypeStr + " - "
			+ errTypeStr + " (" + curItem + "): @ [" 
			+ LexRecordUtil.GetLexRecordInfo(lexRecord) + "]"
			+ errFixStr + GlobalVars.LS_STR;
        errMsg_ += errMsg;

		// update err no
		AddErrTypeNo(errType, errTypeNos);
    }
	/**
	* Get the error message.
	* 
	* @return	String of error message
	*/
    public static String GetErrMsg()
    {
        return errMsg_;
    }
	/**
	* Get the stats of errors.
	* 
	* @param	errTypeStrs 	string of error type	
	* @param	errTypeNos 		no. of error type
	*/
	public static String GetErrStats(String[][] errTypeStrs, int[] errTypeNos)
	{
		String errStatsStr = new String();

		for(int i = 1; i < errTypeNos.length; i++)
		{
			errStatsStr += i + ". " + errTypeStrs[i][ERR] + ": " 
				+ errTypeNos[i] + GlobalVars.LS_STR;
		}
		errStatsStr += "---------------------------" + GlobalVars.LS_STR;
		errStatsStr += errTypeStrs[ERR][ERR_TOTAL] + ": " 
			+ errTypeNos[ERR_TOTAL] + GlobalVars.LS_STR;

		return errStatsStr;
	}

	// private method
	private static void AddErrTypeNo(int errType, int[] errTypeNos)
	{
		if((errType > 0) && (errType < errTypeNos.length))
		{
			errTypeNos[errType]++;		// update err type no
			errTypeNos[ERR_TOTAL]++;	// update total error no
		}
	}

	protected static String GetErrTypeStr(int errType, String[][] errTypeStrs)
	{
		String errTypeStr = errTypeStrs[ERR_NONE][ERR];	
		if((errType >= 0) && (errType < errTypeStrs.length))
		{
			errTypeStr = errTypeStrs[errType][ERR];
		}
		return errTypeStr;
	}

	protected static String GetErrFixStr(int errType, String[][] errTypeStrs)
	{
		String errTypeStr = errTypeStrs[ERR_NONE][FIX];	
		if((errType >= 0) && (errType < errTypeStrs.length))
		{
			errTypeStr = errTypeStrs[errType][FIX];
		}
		return errTypeStr;
	}

    // data member
    protected static String errMsg_ = new String();

	final private static int ERR = 0;
	final private static int FIX = 1;

	final public static int ERR_NONE = 0;	/* No error */
	final public static int ERR_TOTAL = 0;	/* Total number of errors */
}
