package gov.nih.nlm.nls.lexCheck.CheckCont;
import java.util.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
/*****************************************************************************
* This class provides a Java utility class of of a lexRecord.
*
* <p><b>History:</b>
* <ul>
* <li>SCR-39, chlu, 03/28/2013, Add new features to check content
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2013
****************************************************************************/
public class LexRecordUtil
{
	// Get a item from lexRecord
	public static String GetItemFromLexRecord(LexRecord lexRecord,
		int contentType)
	{
		String outItem = new String();
		switch(contentType)
		{
			case CONTENT_CIT:
				outItem = lexRecord.GetBase();
				break;
		}

		return outItem;
	}

	// Get an list from lexRecord
	public static Vector<String> GetListFromLexRecord(LexRecord lexRecord,
		int contentType)
	{
		Vector<String> outList = new Vector<String>();

		switch(contentType)
		{
			case CONTENT_SV:
				outList = lexRecord.GetSpellingVars();
				break;
			case CONTENT_VAR:
				outList = lexRecord.GetVariants();
				break;
			case CONTENT_NOM:
				outList = lexRecord.GetNominalizations();
				break;
			case CONTENT_ABB:
				outList = lexRecord.GetAbbreviations();
				break;
			case CONTENT_ACR:
				outList = lexRecord.GetAcronyms();
				break;
		}

		return outList;
	}

	// Update an item
	public static void SetItemInLexRecord(LexRecord lexRecord, int contentType,
		String item)
	{
		switch(contentType)
		{
			case CONTENT_CIT:
				lexRecord.SetBase(item);
				break;
		}
	}
	
	// Update a list for the specified index
	public static void SetListInLexRecord(LexRecord lexRecord,
		int contentType, Vector<String> inList)
	{
		switch(contentType)
		{
			case CONTENT_SV:
				lexRecord.SetSpellingVars(inList);
				break;
			case CONTENT_VAR:
				lexRecord.SetVariants(inList);
				break;
			case CONTENT_NOM:
				lexRecord.SetNominalizations(inList);
				break;
			case CONTENT_ABB:
				lexRecord.SetAbbreviations(inList);
				break;
			case CONTENT_ACR:
				lexRecord.SetAcronyms(inList);
				break;
		}
	}

	// Update an item in a list for the specified index
	public static void SetItemInListInLexRecordAt(LexRecord lexRecord,
		int contentType, String item, int index)
	{
		switch(contentType)
		{
			case CONTENT_SV:
				lexRecord.GetSpellingVars().setElementAt(item, index);
				break;
			case CONTENT_VAR:
				lexRecord.GetVariants().setElementAt(item, index);
				break;
			case CONTENT_NOM:
				lexRecord.GetNominalizations().setElementAt(item, index);
				break;
			case CONTENT_ABB:
				lexRecord.GetAbbreviations().setElementAt(item, index);
				break;
			case CONTENT_ACR:
				lexRecord.GetAcronyms().setElementAt(item, index);
				break;
		}
	}

	// get the String of content type
	public static String GetContentTypeStr(int contentType)
	{
		String contentTypeStr = contentTypeStrs_[CONTENT_NONE];
		if((contentType > 0) && (contentType <= contentTypeStrs_.length))
		{
			contentTypeStr = contentTypeStrs_[contentType];
		}
		return contentTypeStr;
	}

	// get the String of crossRef type
	public static String GetCrossRefTypeStr(int crossRefType)
	{
		String crossRefTypeStr = crossRefTypeStrs_[CROSS_CHK_NONE];
		if((crossRefType > 0) && (crossRefType <= crossRefTypeStrs_.length))
		{
			crossRefTypeStr = crossRefTypeStrs_[crossRefType];
		}
		return crossRefTypeStr;
	}

	// check if legal category
	public static boolean IsCategory(String inCat)
	{
		boolean validFlag = catStrs_.contains(inCat);
		return validFlag;
	}

	// return category string
	public static String GetCategory(int catIndex)
	{
		String catStr = new String();
		if((catIndex >= 0) && (catIndex < catStrs_.size()))
		{
			catStr = catStrs_.elementAt(catIndex);
		}

		return catStr;
	}

	// get the eui|base|category information of lexRecord
	public static String GetLexRecordInfo(LexRecord lexRecord)
	{
		String eui = lexRecord.GetEui();
		String cat = lexRecord.GetCategory();
		String base = lexRecord.GetBase();

		String info = eui + FS_STR + base + FS_STR + cat;
		return info;
	}

    // data member
	/** field separator */
	public final static String FS_STR = "|";

	// Content type
	public final static int CONTENT_NONE = 0;
	public final static int CONTENT_CIT = 1;
	public final static int CONTENT_SV = 2;
	public final static int CONTENT_CAT = 3;
	public final static int CONTENT_EUI = 4;
	public final static int CONTENT_VAR = 5;
	public final static int CONTENT_NOM = 6;
	public final static int CONTENT_ABB = 7;
	public final static int CONTENT_ACR = 8;
	
	// type string
	private static String[] contentTypeStrs_ = 
	{
		"None",
		"citation",
		"spelling variants",
		"category",
		"EUI",
		"variants",
		"nominalizations",
		"abbreviations",
		"acronyms"
	};

	// Cross check type
	public final static int CROSS_CHK_NONE = 0;
	public final static int CROSS_CHK_DUP_EUI = 1;
	public final static int CROSS_CHK_DUP_REC = 2;
	public final static int CROSS_CHK_NOM = 3;
	public final static int CROSS_CHK_ABB = 4;
	public final static int CROSS_CHK_ACR = 5;
	
	// type string
	private static String[] crossRefTypeStrs_ = 
	{
		"None",
		"duplicate EUIs",
		"duplicate LexRecords",
		"nominalizations",
		"abbreviations",
		"acronyms"
	};
	// category
	public final static int CAT_ADJ = 0;
	public final static int CAT_ADV = 1;
	public final static int CAT_AUX = 2;
	public final static int CAT_COMPL = 3;
	public final static int CAT_CONJ = 4;
	public final static int CAT_DET = 5;
	public final static int CAT_MODAL = 6;
	public final static int CAT_NOUN = 7;
	public final static int CAT_PREP = 8;
	public final static int CAT_PRON = 9;
	public final static int CAT_VERB = 10;

	private static Vector<String> catStrs_ = new Vector<String>();
	static
	{
		catStrs_.addElement("adj");
		catStrs_.addElement("adv");
		catStrs_.addElement("aux");
		catStrs_.addElement("compl");
		catStrs_.addElement("conj");
		catStrs_.addElement("det");
		catStrs_.addElement("modal");
		catStrs_.addElement("noun");
		catStrs_.addElement("prep");
		catStrs_.addElement("pron");
		catStrs_.addElement("verb");
	}
}
