package gov.nih.nlm.nls.lexCheck.Compl;
import java.util.*;
/*****************************************************************************
* This class checks prepositional phrase complement format.
*
* <p><b>History:</b>
* <ul>
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2013
****************************************************************************/
public class CheckPphr
{
    // public method
    /**
    * Check if the filler is a legal prepositional phrase complement format
    *
    * @param   filler  the filler for checking
    *
    * @return  true or false for a legal or illegal format of prepositional 
    *          phrase complement
    */
    public static boolean IsLegal(String filler) 
    {
        boolean flag = true;
        // check the starting string "pphr(" and the ending string ")"
        if((filler.startsWith("pphr(") == false)
        || (filler.endsWith(")") == false))
        {
            return false;
        }
        // check first preposition
        int index = filler.indexOf(",");
        if(index <= 5)        // "pphr("
        {
            return false;
        }
        String firstPreposition = filler.substring(5, index);
        if(CheckPreposition.IsLegal(firstPreposition) == false)
        {
            return false;
        }
        // check second argument
        int endIndex = filler.lastIndexOf(")");
        int comma2Index = filler.indexOf(",pphr", index+1);
        // check second argument
        if(comma2Index == -1) // no 2nd pphr
        {
            String argument = filler.substring(index+1, endIndex);
            flag = CheckArgument(argument);
        }
        else // with 2nd pphr
        {
            String pphr2 = filler.substring(comma2Index+1, endIndex);
            flag = CheckPphr.IsLegal(pphr2);
        }
        return flag;
    }
    // private method
    // Check second argument
    private static boolean CheckArgument(String argument)
    {
        boolean flag = false;
        // adj, advbl, np, whfincomp
        if(argument_.contains(argument) == true)
        {
            flag = true;
        }
        else
        {
            if((argument.startsWith("binfcomp:") == true)
            || (argument.startsWith("edcomp:") == true)
            || (argument.startsWith("infcomp:") == true)
            || (argument.startsWith("ingcomp:") == true)
            || (argument.startsWith("whinfcomp:") == true))
            {
                int index = argument.indexOf(":");
                String interpretation = argument.substring(index+1);
                flag = CheckInterpretation.IsLegal(interpretation);
            }
            else if((argument.startsWith("np|") == true)
            && (argument.endsWith("|") == true)
            && (argument.equals("np|") == false))
            {
                flag = 
                    ((argument.indexOf("|") + 1) != argument.lastIndexOf("|"));
            }
        }
        return flag;
    }
    // data member
    private static HashSet<String> argument_ = new HashSet<String>(4);
    static
    {
        argument_.add("adj");
        argument_.add("advbl");
        argument_.add("np");
        argument_.add("whfincomp");
    }
}
