package gov.nih.nlm.nls.lexCheck.Lib;
import java.util.*;
/*****************************************************************************
* This class provides a Java object class for adverb entry.
*
* <p><b>History:</b>
* <ul>
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2013
****************************************************************************/
public class AdvEntry
{
    /**
    *    Creates a default adverb entry object
    */
    public AdvEntry()
    {
        variants_ = new Vector<String>();
        interrogative_ = false;
        modification_ = new Vector<String>();
        negative_ = null;
    }
    // public method
    /**
    * Get the collection of inflectional variants of an adverb entry
    *
    * @return  a Vector with inflectional variant elements.
    */
    public Vector<String> GetVariants()
    {
        return variants_;
    }
    /**
    * Get the status of interrogative of an adverb entry
    *
    * @return  the status of interrogative
    */
    public boolean IsInterrogative()
    {
        return interrogative_;
    }
    /**
    * Get the collection of modifications of an adverb entry
    *
    * @return  a Vector with modification elements.
    */
    public Vector<String> GetModification()
    {
        return modification_;
    }
    /**
    * Get the state of negative of an adverb entry
    *
    * @return  the state of negative
    */
    public String GetNegative()
    {
        return negative_;
    }
    /**
    * Add an inflectional variant to the collection of inflectional variants 
    * of an adverb entry.
    *
    * @param  variant inflectional variant to be added.
    */
    public void AddVariant(String variant)
    {
        variants_.add(variant);
    }
    /**
    * Set the collection of inflectional variants of an adverb entry
    *
    * @param  variants Vector of inflectional variants to be set.
    */
    public void SetVariants(Vector<String> variants)
    {
        variants_ = variants;
    }
    /**
    * Set the status of interrogative of an adverb entry
    *
    * @param  interrogative status of interrogative
    */
    public void SetInterrogative(boolean interrogative)
    {
        interrogative_ = interrogative;
    }
    /**
    * Add an modification type to the collection of inflectional variants 
    * of an adverb entry.
    *
    * @param  modification modification type to be added.
    */
    public void AddModification(String modification)
    {
        modification_.add(modification);
    }
    /**
    * Set the modification type of an adverb entry
    *
    * @param  modification modification type
    */
    public void SetModification(Vector<String> modification)
    {
        modification_ = modification;
    }
    /**
    * Set the state of negative of an adverb entry
    *
    * @param  negative state of negative
    */
    public void SetNegative(String negative)
    {
        negative_ = negative;
    }
    /**
    * Get the text format of the adverb entry
    *
    * @return  text format of the adverb entry
    */
    public String GetText()
    {
        String text = new String();
        text = TextLib.AddToText(text, "variants=", variants_, 1);
        text = TextLib.AddToText(text, "interrogative", interrogative_, 1);
        text = TextLib.AddToText(text, "modification_type=", 
            modification_, 1);
        text = TextLib.AddToText(text, "", negative_, 1);
        return text;
    }
    /**
    * Get the xml format of the adverb entry
    *
    * @return  xml format of the adverb entry
    */
    public String GetXml()
    {
        boolean convertFlag = true;
        String xml = new String();
        xml = XmlLib.AddToXml(xml, "<advEntry>", 2);
        xml = XmlLib.AddToXml(xml, "<variants>", "</variants>",
            variants_, 3, convertFlag);
        xml = XmlLib.AddToXml(xml, "<modification>", "</modification>",
            modification_, 3, convertFlag);
        if(negative_ != null)
        {
            String negativeTag = "<negative type=\"" + negative_ + "\"/>";
            xml = XmlLib.AddToXml(xml, negativeTag, 3);
        }
        xml = XmlLib.AddToXml(xml, "<interrogative/>", interrogative_, 3);
        xml = XmlLib.AddToXml(xml, "</advEntry>", 2); 
        return xml;
    }
    // data member
    private Vector<String> variants_ = new Vector<String>();
    private boolean interrogative_ = false;
    private Vector<String> modification_ = new Vector<String>();
    private String negative_ = null;
}
