package gov.nih.nlm.nls.lexCheck.Lib;
/*****************************************************************************
* This class provides a class of handling error message for lexCheck.
*
* <p><b>History:</b>
*
* @author NLM Lexical System Development Team
*
* @version    V-2013
****************************************************************************/
public class ErrMsg
{
    // public method
    /**
    * Print out the error message.
    *
    * @param   printFlag   the flag for print out check result
    * @param   index          the index of error message to print out
    * @param   lineObject  the java object of the input line
    * @param   token       the token string of the problem
    * @param   beginIndex  the index of the beginning mark of the problem
    * @param   endIndex    the index of the ending mark of the problem
    * @param   isTab       the flag of tab strating
    */
    public static void PrintErrMsg(boolean printFlag, int index, 
        LineObject lineObject, String token, int beginIndex, int endIndex, 
        boolean isTab)
    {
        if((printFlag == true) && (index != NONE))
        {
            curErrMsg_ = 
                GetMsg(index, lineObject, token, beginIndex, endIndex, isTab);
            System.out.println(curErrMsg_);
        }
    }
    /**
    * Get the crrent error message.
    */
    public static String GetErrMsg()
    {
        return curErrMsg_;
    }
    /**
    * Set the crrent error message.
    *
    * @param   errMsg   error message to be set
    */
    public static void SetErrMsg(String errMsg)
    {
        curErrMsg_ = errMsg;
    }
    /**
    * Reset the crrent error message to a new string.
    */
    public static void ResetErrMsg()
    {
        curErrMsg_ = new String();
    }
    // private methods
    private static String GetMsg(int index, LineObject lineObject, String token,
        int beginIndex, int endIndex, boolean isTab)
    {
        String line = lineObject.GetLine();
        int lineNum = lineObject.GetLineNum();
        String msg = new String(); 
        if(token == null)
        {
            msg = "Syntax Error: at the beginning of line " + lineNum + LS
                + " Rule-" + (index+1) + ": " + errMsg_[index] + LS
                + line + LS
                + GetErrorPositionString(beginIndex, endIndex, isTab);
        }
        else
        {
            msg = "Syntax Error: at token '" + token + "' on line " + lineNum 
                + LS + "  Rule-" + (index+1) + ": " + errMsg_[index] + LS 
                + line + LS 
                + GetErrorPositionString(beginIndex, endIndex, isTab);
        }
        return msg;
    }
    // print a line with marker ^ to point out the position of errors
    private static String GetErrorPositionString(int beginIndex, int endIndex,
        boolean isTab)
    {
        // Refine beginIndex & endIndex
        if(beginIndex > endIndex)
        {
            int tempIndex = beginIndex;
            beginIndex = endIndex;
            endIndex = tempIndex;
        }
        if(beginIndex < 0)
        {
            beginIndex = 0;
        }
        if(endIndex < 0)
        {
            endIndex = 0;
        }
        // printout the tab & marker
        StringBuffer positionStr = new StringBuffer();
        int startIndex = 0;
        if(isTab == true)
        {
            positionStr.append('\t');
            startIndex = 1;
        }
        // add space to stringBuffer
        for(int i = startIndex; i < endIndex+1; i++)
        {
            positionStr.append(' ');
        }
        positionStr.setCharAt(beginIndex, '^');
        positionStr.setCharAt(endIndex, '^');
        return positionStr.toString();
    }
    // final data
    /** line separator */
    final public static String LS = System.getProperty("line.separator");
    /** error message: none */
    final public static int NONE = -1;
    /** error message: filler format error */
    final public static int FILLER_FORMAT_ERR = 0;
    /** error message: filler blank error */
    final public static int FILLER_BLANK_ERR = 1;
    /** error message: filler starting char error */
    final public static int FILLER_START_CHAR_ERR = 2;
    /** error message: base slot eorror */
    final public static int BASE_SLOT_ERR = 3;
    /** error message: spelling variant slot error */
    final public static int SPELLING_VAR_SLOT_ERR = 4;
    /** error message: entry slot error */
    final public static int ENTRY_SLOT_ERR = 5;
    /** error message: entry filler error */
    final public static int ENTRY_FILLER_ERR = 6;
    /** error message: tab error */
    final public static int TAB_ERR = 7;
    /** error message: category slot error */
    final public static int CAT_SLOT_ERR = 8;
    /** error message: category filler error */
    final public static int CAT_FILLER_ERR = 9;
    /** error message: variants error */
    final public static int VARIANTS_ERR = 10;
    /** error message: variants format error */
    final public static int VARIANTS_FORMAT_ERR = 11;
    /** error message: annotation slot error */
    final public static int ANNOTATION_SLOT_ERR = 12;
    /** error message: signature slot error */
    final public static int SIGNATURE_SLOT_ERR = 13;
    /** error message: end slot error */
    final public static int END_SLOT_ERR = 14;
    /** error message: end code error */
    final public static int END_CODE_ERR = 15;
    /** error message: abbreviation slot error */
    final public static int ABBREVIATION_SLOT_ERR = 16;
    /** error message: abbreviation filler error */
    final public static int ABBREVIATION_FILLER_ERR = 17;
    /** error message: acronym slot error */
    final public static int ACRONYM_SLOT_ERR = 18;
    /** error message: acronym slot filler error */
    final public static int ACRONYM_FILLER_ERR = 19;
    /** error message: suplicate slot error */
    final public static int DUPLICATE_SLOT_ERR = 20;
    /** error message: determiner variants slot error */
    final public static int DET_VAR_SLOT_ERR = 21;
    /** error message: determiner variants filler error */
    final public static int DET_VAR_FILLER_ERR = 22;
    /** error message: determiner interrogative slot error */
    final public static int DET_INTERROGATIVE_SLOT_ERR = 23;
    /** error message: determiner demonstrative slot error */
    final public static int DET_DEMONSTRATIVE_SLOT_ERR = 24;
    /** error message: pronoun variant slot error */
    final public static int PRON_VAR_SLOT_ERR = 25;
    /** error message: pronoun variant filler error */
    final public static int PRON_VAR_FILLER_ERR = 26;
    /** error message: pronoun gender slot error */
    final public static int PRON_GENDER_SLOT_ERR = 27;
    /** error message: pronoun gender filler error */
    final public static int PRON_GENDER_FILLER_ERR = 28;
    /** error message: pronoun interrogative slot error */
    final public static int PRON_INTERROGATIVE_SLOT_ERR = 29;
    /** error message: pronoun type slot error */
    final public static int PRON_TYPE_SLOT_ERR = 30;
    /** error message: pronoun type filler error */
    final public static int PRON_TYPE_FILLER_ERR = 31;
    /** error message: modal variants slot error */
    final public static int MODAL_VAR_SLOT_ERR = 32;
    /** error message: modal variants filler error */
    final public static int MODAL_VAR_FILLER_ERR = 33;
    /** error message: auxiliary variants slot error */
    final public static int AUX_VAR_SLOT_ERR = 34;
    /** error message: auxiliary variants filler error */
    final public static int AUX_VAR_FILLER_ERR = 35;
    /** error message: adverb variants slot error */
    final public static int ADV_VAR_SLOT_ERR = 36;
    /** error message: adverb variants filler error */
    final public static int ADV_VAR_FILLER_ERR = 37;
    /** error message: adverb interrogative slot error */
    final public static int ADV_INTERROGATIVE_SLOT_ERR = 38;
    /** error message: adverb modification slot error */
    final public static int ADV_MODIFICATION_SLOT_ERR = 39;
    /** error message: adverb modification filler error */
    final public static int ADV_MODIFICATION_FILLER_ERR = 40;
    /** error message: adverb negative slot error */
    final public static int ADV_NEGATIVE_SLOT_ERR = 41;
    /** error message: adverb broad negative slot error */
    final public static int ADV_BROAD_NEGATIVE_SLOT_ERR = 42;
    /** error message: adjective variants slot error */
    final public static int ADJ_VAR_SLOT_ERR = 43;
    /** error message: adjective variants filler error */
    final public static int ADJ_VAR_FILLER_ERR = 44;
    /** error message: adjective position slot error */
    final public static int ADJ_POSITION_SLOT_ERR = 45;
    /** error message: adjective position filler error */
    final public static int ADJ_POSITION_FILLER_ERR = 46;
    /** error message: adjective complement slot error */
    final public static int ADJ_COMPL_SLOT_ERR = 47;
    /** error message: adjective compelment filler error */
    final public static int ADJ_COMPL_FILLER_ERR = 48;
    /** error message: adjective stative slot error */
    final public static int ADJ_STATIVE_SLOT_ERR = 49;
    /** error message: adjective nominalization slot error */
    final public static int ADJ_NOMINALIZATION_SLOT_ERR = 50;
    /** error message: adjective nominalization filler error */
    final public static int ADJ_NOMINALIZATION_FILLER_ERR = 51;
    /** error message: noun variants slot error */
    final public static int NOUN_VAR_SLOT_ERR = 52;
    /** error message: noun variants filler error */
    final public static int NOUN_VAR_FILLER_ERR = 53;
    /** error message: none complement slot error */
    final public static int NOUN_COMPL_SLOT_ERR = 54;
    /** error message: noun complement filler error */
    final public static int NOUN_COMPL_FILLER_ERR = 55;
    /** error message: noun nominalization slot error */
    final public static int NOUN_NOMINALIZATION_SLOT_ERR = 56;
    /** error message: noun nominalization filler error */
    final public static int NOUN_NOMINALIZATION_FILLER_ERR = 57;
    /** error message: noun tradename slot error */
    final public static int NOUN_TRADENAME_SLOT_ERR = 58;
    /** error message: noun tradename filler error */
    final public static int NOUN_TRADENAME_FILLER_ERR = 59;
    /** error message: noun trademark slot error */
    final public static int NOUN_TRADEMARK_SLOT_ERR = 60;
    /** error message: noun trademark filler error */
    final public static int NOUN_TRADEMARK_FILLER_ERR = 61;
    /** error message: noun proper slot error */
    final public static int NOUN_PROPER_SLOT_ERR = 62;
    /** error message: verb variant slot error */
    final public static int VERB_VAR_SLOT_ERR = 63;
    /** error message: verb variants filler error */
    final public static int VERB_VAR_FILLER_ERR = 64;
    /** error message: verb intransitive slot error */
    final public static int VERB_INTRAN_SLOT_ERR = 65;
    /** error message: verb intransitive filler error */
    final public static int VERB_INTRAN_FILLER_ERR = 66;
    /** error message: verb intransitive slot error */
    final public static int VERB_INTRAN2_SLOT_ERR = 67;
    /** error message: verb intransitive filler error */
    final public static int VERB_INTRAN2_FILLER_ERR = 68;
    /** error message: verb transitive slot error */
    final public static int VERB_TRAN_SLOT_ERR = 69;
    /** error message: verb transitive filler error */
    final public static int VERB_TRAN_FILLER_ERR = 70;
    /** error message: verb ditransitive slot error */
    final public static int VERB_DITRAN_SLOT_ERR = 71;
    /** error message: verb ditransitive filler error */
    final public static int VERB_DITRAN_FILLER_ERR = 72;
    /** error message: verb linking slot error */
    final public static int VERB_LINK_SLOT_ERR = 73;
    /** error message: verb linking filler error */
    final public static int VERB_LINK_FILLER_ERR = 74;
    /** error message: verb complex transitive slot error */
    final public static int VERB_CPLXTRAN_SLOT_ERR = 75;
    /** error message: verb complex transitive filler error */
    final public static int VERB_CPLXTRAN_FILLER_ERR = 76;
    /** error message: verb nominalization slot error */
    final public static int VERB_NOMINALIZATION_SLOT_ERR = 77;
    /** error message: verb nominalization filler error */
    final public static int VERB_NOMINALIZATION_FILLER_ERR = 78;
    /** error message: verb complement number error */
    final public static int VERB_COMPL_NUM_ERR = 79;
    /** error message: slot filler error */
    final public static int SLOT_FILLER_ERR = 80;
    /** error message: slot fillernumber error */
    final public static int SLOT_FILLER_NUMBER_ERR = 81;
    /** error message: slot number error */
    final public static int SLOT_NUMBER_ERR = 82;
    // data member
    private static String[] errMsg_ = {
        "The filler has an illegal format",
        "A blank filler on the right side of slot= is not allowed", 
        "The first character of the filler is illegal", 
        "A lexical record begins with '{base='",
        "A line begins with 'spelling_variant=' is expected",
        "A line begins with 'entry=' is expected",
        "The Eui is a seven number preceded by the letter \"E\"", 
        "A line begins with tab is expected",
        "A line begins with '\tcat=' is expected",
        "The category is illegal", 
        "'\tvariants=' slot is missing or has illegal format",
        "The filler of 'variants=' is illegal", 
        "A line begins with 'annotation=' is expected",
        "A line begins with 'signature=' is expected",
        "A line begins with '}' is expected",
        "A lexical record ends with '}'",
        "A line begins with '\tabbreviation_of=' is expected",
        "The format of abbreviation is illegal", 
        "A line begins with '\tacronym_of=' is expected",
        "The format of acronym is illegal", 
        "The slot= is duplicated", 
        "'\tvariants=' slot is missing or has illegal format",
        "The filler of 'variants=' is illegal", 
        "A line begins with '\tinterrogative' is expected",
        "A line begins with '\tdemonstrative' is expected",
        "'\tvariants=' slot is missing or has illegal format",
        "The filler of 'variants=' is illegal", 
        "'\tgender=' slot is missing or has illegal format",
        "The filler of 'gender=' is illegal", 
        "A line begins with '\tinterrogative' is expected",
        "'\ttype=' slot is missing or has illegal format",
        "The filler of 'type=' is illegal", 
        "'\tvariant=' slot is missing or has illegal format",
        "The filler of 'variant=' is illegal", 
        "'\tvariant=' slot is missing or has illegal format",
        "The filler of 'variant=' is illegal", 
        "'\tvariants=' slot is missing or has illegal format",
        "The filler of '\tvariants=' is illegal", 
        "'\tinterrogative' slot is missing or has illegal format",
        "'\tmodification_type=' slot is missing or has illegal format",
        "The filler of '\tmodification_type=' is illegal", 
        "'\tnegative' slot is missing or has illegal format",
        "'\tbroad_negative' slot is missing or has illegal format",
        "'\tvariants=' slot is missing or has illegal format",
        "The filler of '\tvariants=' is illegal", 
        
        "'\tposition=' slot is missing or has illegal format",
        "The filler of '\tposition=' is illegal", 
        "'\tcompl=' slot is missing or has illegal format",
        "The filler of '\tcompl=' is illegal", 
        "'\tstative=' slot is missing or has illegal format",
        "'\tnominalization=' slot is missing or has illegal format",
        "The filler of '\tnominalization=' is illegal", 
        "'\tvariants=' slot is missing or has illegal format",
        "The filler of '\tvariants=' is illegal", 
        "'\tcompl=' slot is missing or has illegal format",
        "The filler of '\tcompl=' is illegal", 
        "'\tnominalization_of=' slot is missing or has illegal format",
        "The filler of '\tnominalization_of=' is illegal", 
        "'\ttrademark=' slot is missing or has illegal format",
        "The filler of '\ttrademark=' is illegal", 
        "'\ttrademark' slot is missing or has illegal format",
        "The filler of '\ttrademark' is illegal", 
        "'\tproper=' slot is missing or has illegal format",
        "'\tvariants=' slot is missing or has illegal format",
        "The filler of '\tvariants=' is illegal", 
        "'\tintran' slot is missing or has illegal format",
        "The filler of '\tintran' is illegal", 
        "'\tintran;part( )' slot is missing or has illegal format",
        "The filler of '\tintran;part( )' is illegal", 
        "'\ttran=' slot is missing or has illegal format",
        "The filler of '\ttran=' is illegal", 
        "'\tditran=' slot is missing or has illegal format",
        "The filler of '\tditran=' is illegal", 
        "'\tlink=' slot is missing or has illegal format",
        "The filler of '\tlink=' is illegal", 
        "'\tcplxtran=' slot is missing or has illegal format",
        "The filler of '\tcplxtran=' is illegal", 
        "'\tnominalization=' slot is missing or has illegal format",
        "The filler of '\tnominalization=' is illegal", 
        "The number of verb complement is 0 ( at least 1)", 
        "Can't find delimiter in CheckSlotFiller()", 
        "More than one slot=filler exists", 
        "More than one slot exists", 
        };
    private static String curErrMsg_ = new String();
}
