package gov.nih.nlm.nls.lexCheck.Lib;
import java.util.*;
/*****************************************************************************
* This class provides a Java object for a lexical record.
*
* <p><b>History:</b>
* <ul>
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2013
****************************************************************************/
public class LexRecord
{
    /**
    *    Creates a default lexical record object
    */
    public LexRecord()
    {
        base_ = null;
        spellingVars_ = new Vector<String>();
        eui_ = null;
        category_ = null;
        catEntry_ = null;
        acronyms_ = new Vector<String>();
        abbreviations_ = new Vector<String>();
        annotations_ = new Vector<String>();
        signature_ = null;
        end_ = "}";
    }
    // public method
    /**
    * set the lexical record to a given lexRecord
    */
    public void SetLexRecord(LexRecord lexRecord)
    {
        base_ = lexRecord.GetBase();
        spellingVars_ = lexRecord.GetSpellingVars();
        eui_ = lexRecord.GetEui();
        category_ = lexRecord.GetCategory();
        catEntry_ = lexRecord.GetCatEntry();
        acronyms_ = lexRecord.GetAcronyms();
        abbreviations_ = lexRecord.GetAbbreviations();
        annotations_ = lexRecord.GetAnnotations();
        signature_ = lexRecord.GetSignature();
        end_ = lexRecord.GetEnd();
    }
    /**
    * Reset the lexical record
    */
    public void Reset()
    {
        base_ = new String();
        spellingVars_ = new Vector<String>();
        eui_ = null;
        category_ = new String();
        catEntry_ = null;
        acronyms_ = new Vector<String>();
        abbreviations_ = new Vector<String>();
        annotations_ = new Vector<String>();
        signature_ = null;
        end_ = "}";
    }
    /**
    * Get the xml format of a lexical record with root of lexRecords
    *
    * @return  xml format of the lexical record
    */
    public String GetXml()
    {
        return GetXml(0);
    }
    /**
    * Get the xml format of a lexical records begin tag
    *
    * @return  xml begin tag of the lexical records
    */
    public static String GetXmlRootBeginTag()
    {
        String xml = "<" + xmlRootTag_ + ">";
        return xml;
    }
    /**
    * Get the xml format of a lexical records begin tag
    *
    * @return  xml begin tag of the lexical records
    */
    public static String GetXmlRootEndTag()
    {
        String xml = "</" + xmlRootTag_ + ">";
        return xml;
    }
    /**
    * Get the xml format of a lexical record with root of lexRecord
    *
    * @return  xml format of the lexical record
    */
    public String GetXml(int indLevel)
    {
        boolean convertFlag = true;
        String xml = new String();
        xml = XmlLib.AddToXml(xml, "<lexRecord>", indLevel);
        xml = XmlLib.AddToXml(xml, "<base>", "</base>", base_, indLevel+1,
            convertFlag);
        xml = XmlLib.AddToXml(xml, "<eui>", "</eui>", eui_, indLevel+1,
            convertFlag);
        xml = XmlLib.AddToXml(xml, "<cat>", "</cat>", category_, indLevel+1,
            convertFlag);
        xml = XmlLib.AddToXml(xml, "<spellingVars>", "</spellingVars>", 
            spellingVars_, indLevel+1, convertFlag);
        xml += InflVarsAndAgreements.GetXml(this);
        xml += catEntry_.GetXml(category_);
        xml = XmlLib.AddToXml(xml, "<acronyms>", "</acronyms>", acronyms_, 
            indLevel+1, convertFlag);
        xml = XmlLib.AddToXml(xml, "<abbreviations>", "</abbreviations>", 
            abbreviations_, indLevel+1, convertFlag);
        xml = XmlLib.AddToXml(xml, "<annotations>", "</annotations>", 
            annotations_, indLevel+1, convertFlag);
        xml = XmlLib.AddToXml(xml, "<signature>", "</signature>", signature_, 
            indLevel+1, convertFlag);
        xml = XmlLib.AddToXml(xml, "</lexRecord>", indLevel);
        return xml;
    }
    /**
    * Get the text format of this lexical object
    *
    * @return  text format of this lexical object
    */
    public String GetText()
    {
        String text = new String();
        text = TextLib.AddToText(text, "{base=", base_, 0);
        text = TextLib.AddToText(text, "spelling_variant=", spellingVars_, 0);
        text = TextLib.AddToText(text, "entry=", eui_, 0);
        text = TextLib.AddToText(text, "cat=", category_, 1);
        // get text from category entry
        text += catEntry_.GetText(category_);
        text = TextLib.AddToText(text, "acronym_of=", acronyms_, 1);
        text = TextLib.AddToText(text, "abbreviation_of=", abbreviations_, 1);
        text = TextLib.AddToText(text, "annotation=", annotations_, 0);
        text = TextLib.AddToText(text, "signature=", signature_, 0);
        text = TextLib.AddToText(text, "", end_, 0);
        return text;
    }
    /**
    * Get the release format of this lexical object
    * 1. remove annotation
    * 2. remove signature
    *
    * @return  text format of this lexical object
    */
    public String GetReleaseFormatText()
    {
        String text = new String();
        text = TextLib.AddToText(text, "{base=", base_, 0);
        text = TextLib.AddToText(text, "spelling_variant=", spellingVars_, 0);
        text = TextLib.AddToText(text, "entry=", eui_, 0);
        text = TextLib.AddToText(text, "cat=", category_, 1);
        // get text from category entry
        text += catEntry_.GetText(category_);
        text = TextLib.AddToText(text, "acronym_of=", acronyms_, 1);
        text = TextLib.AddToText(text, "abbreviation_of=", abbreviations_, 1);
        //text = TextLib.AddToText(text, "annotation=", annotations_, 0);
        //text = TextLib.AddToText(text, "signature=", signature_, 0);
        text = TextLib.AddToText(text, "", end_, 0);
        return text;
    }
    /**
    * Get the fixed text of this lexical object.  The steps are:
    * <ol>
    * <li> add base into irreg
    * <li> rm signature from lexRecord
    * <li> add eui|base|cat|cSig|mSig|aSig|lastAct|cDate|mDate|aDate|lexRecord
    * </ol>
    * This format is used as the input of LexBuild
    *
    * @return  text format of this lexical object
    */
    public String GetDbFormatText(String cDate)
    {
        String SP = "|";
        String cSig = signature_;
        String mSig = signature_;
        String aSig = "browne";
        int lastAct = 0;    // Old LexBuild System
        String text = new String();
        text = eui_ + SP + base_ + SP + category_ + SP + cSig + SP
            + mSig + SP + aSig + SP + lastAct + SP + cDate + SP 
            + cDate + SP + cDate + SP;
        text = TextLib.AddToText(text, "{base=", base_, 0);
        text = TextLib.AddToText(text, "spelling_variant=", spellingVars_, 0);
        text = TextLib.AddToText(text, "entry=", eui_, 0);
        text = TextLib.AddToText(text, "cat=", category_, 1);
        // get text from category entry
        text += catEntry_.GetText(category_);
        text = TextLib.AddToText(text, "acronym_of=", acronyms_, 1);
        text = TextLib.AddToText(text, "abbreviation_of=", abbreviations_, 1);
        //text = TextLib.AddToText(text, "annotation=", annotations_, 0);
        //text = TextLib.AddToText(text, "signature=", signature_, 0);
        text = TextLib.AddToText(text, "", end_, 0);
        return text;
    }
    /**
    * Get the derived inflectional variants of this lexical record
    *
    * @return  Java object of InflVarsAndAgreements
    */
    public InflVarsAndAgreements GetInflVarsAndAgreements()
    {
        return new InflVarsAndAgreements(this);
    }
    /**
    * print out the text and xml for this lexical record
    */
    public void Print()
    {
        System.out.print(GetText());
        System.out.print(GetXml());
    }
    /**
    * print out the text for the release lexical record
    */
    public void PrintReleaseFormatText()
    {
        System.out.print(GetReleaseFormatText());
    }
    /**
    * print out the text for the database format lexical record
    */
    public void PrintDbFormatText(String cDate)
    {
        System.out.print(GetDbFormatText(cDate));
    }
    /**
    * print out the text for this lexical record
    */
    public void PrintText()
    {
        System.out.print(GetText());
    }
    /**
    * print out the xml for this lexical record
    */
    public void PrintXml()
    {
        System.out.print(GetXml());
    }
    /**
    * Get the base of this lexical record.
    *
    * @return  base in String
    */
    public String GetBase() 
    {
        return base_;
    }
    /**
    * Get the Eui of this lexical record.
    *
    * @return  Eui in String
    */
    public String GetEui() 
    {
        return eui_;
    }
    /**
    * Get the spelling variants of this lexical record.
    *
    * @return  a Vector collection of spelling variants
    */
    public Vector<String> GetSpellingVars()
    {
        return spellingVars_;
    }
    /**
    * Get the acronyms of this lexical record.
    *
    * @return  a Vector collection of acronyms
    */
    public Vector<String> GetAcronyms() 
    {
        return acronyms_;
    }
    /**
    * Get the abbreviations of this lexical record.
    *
    * @return  a Vector collection of abbreviations
    */
    public Vector<String> GetAbbreviations() 
    {
        return abbreviations_;
    }
    /**
    * Get the nominalization of this lexical record.
    * Return an empty vector if category is not verb, noun, or adj
    *
    * @return  a Vector collection of abbreviations
    */
    public Vector<String> GetNominalizations() 
    {
        Vector<String> nominalizations = new Vector<String>();
        
        // get nominalization
        if(category_.equals("noun") == true)
        {
            nominalizations = catEntry_.GetNounEntry().GetNominalization();
        }
        else if(category_.equals("verb") == true)
        {
            nominalizations = catEntry_.GetVerbEntry().GetNominalization();
        }
        else if(category_.equals("adj") == true)
        {
            nominalizations = catEntry_.GetAdjEntry().GetNominalization();
        }
        return nominalizations;
    }
    /**
    * Get the variants of this lexical record.
    * Return an empty Vector for the following:
    * - Only one variant if category is det
    * - No variants if category is prep, conj, compl
    *
    * @return  a Vector collection of variant(s)
    */
    public Vector<String> GetVariants() 
    {
        Vector<String> variants = new Vector<String>();
        // get variants: no for det, prep, conj, and compl
        if(category_.equals("noun") == true)
        {
            variants = catEntry_.GetNounEntry().GetVariants();
        }
        else if(category_.equals("verb") == true)
        {
            variants = catEntry_.GetVerbEntry().GetVariants();
        }
        else if(category_.equals("adj") == true)
        {
            variants = catEntry_.GetAdjEntry().GetVariants();
        }
        else if(category_.equals("adv") == true)
        {
            variants = catEntry_.GetAdvEntry().GetVariants();
        }
        else if(category_.equals("pron") == true)
        {
            variants = catEntry_.GetPronEntry().GetVariants();
        }
        else if(category_.equals("modal") == true)
        {
            variants = catEntry_.GetModalEntry().GetVariant();
        }
        else if(category_.equals("aux") == true)
        {
            variants = catEntry_.GetAuxEntry().GetVariant();
        }
        return variants;
    }
    /**
    * Get the annotations of this lexical record.
    *
    * @return  a Vector collection of annotations
    */
    public Vector<String> GetAnnotations() 
    {
        return annotations_;
    }
    /**
    * Get the signature of this lexical record.
    *
    * @return  signature in String
    */
    public String GetSignature() 
    {
        return signature_;
    }
    /**
    * Get the category name of this lexical record.
    *
    * @return  category name in String
    */
    public String GetCategory() 
    {
        return category_;
    }
    /**
    * Get the record end curly bracket of this lexical record.
    *
    * @return  record end curly bracket
    */
    public String GetEnd() 
    {
        return end_;
    }
    /**
    * Set the base for this lexical record
    *
    * @param  base   the base form
    */
    public void SetBase(String base) 
    {
        base_ = base;
    }
    /**
    * Set the Eui for this lexical record
    *
    * @param  eui   the entry unique identifier
    */
    public void SetEui(String eui) 
    {
        eui_ = eui;
    }
    /**
    * Set (add) a spelling variant for this lexical record
    *
    * @param  spellingVar   spelling variant to be added
    */
    public void SetSpellingVar(String spellingVar) 
    {
        spellingVars_.add(spellingVar);
    }
    /**
    * Set the entire collection of spelling variants for this lexical record
    *
    * @param  spellingVars   the whole collection of spelling variants
    */
    public void SetSpellingVars(Vector<String> spellingVars) 
    {
        spellingVars_ = spellingVars;
    }
    /**
    * Set (add) an acronym for this lexical record
    *
    * @param  acronym   acronym to be added
    */
    public void SetAcronym(String acronym) 
    {
        acronyms_.add(acronym);
    }
    /**
    * Set (add) an abbreviation for this lexical record
    *
    * @param  abbreviation   abbreviation to be added
    */
    public void SetAbbreviation(String abbreviation) 
    {
        abbreviations_.add(abbreviation);
    }
    /**
    * Set (add) an annotation for this lexical record
    *
    * @param  annotation   annotation to be added
    */
    public void SetAnnotation(String annotation) 
    {
        annotations_.add(annotation);
    }
    /**
    * Set the entire collection of acronyms for this lexical record
    *
    * @param  acronyms   the whole collection of acronyms to be set
    */
    public void SetAcronyms(Vector<String> acronyms)
    {
        acronyms_ = acronyms;
    }
    /**
    * Set the entire collection of abbreviations for this lexical record
    *
    * @param  abbreviations   the whole collection of abbreviations to be set
    */
    public void SetAbbreviations(Vector<String> abbreviations)
    {
        abbreviations_ = abbreviations;
    }
    /**
    * Set the entire collection of nominalizations for this lexical record
    *
    * @param  nominalizations   the whole list of nominalizations to be set
    */
    public void SetNominalizations(Vector<String> nominalizations)
    {
        // update nominalization
        if(category_.equals("noun") == true)
        {
            catEntry_.GetNounEntry().SetNominalization(nominalizations);
        }
        else if(category_.equals("verb") == true)
        {
            catEntry_.GetVerbEntry().SetNominalization(nominalizations);
        }
        else if(category_.equals("adj") == true)
        {
            catEntry_.GetAdjEntry().SetNominalization(nominalizations);
        }
    }
    /**
    * Set the entire collection of variants for this lexical record.
    * Do nothing if category is det, prep, conj, compl
    *
    * @param  variants   the whole collection of variants to be set
    */
    public void SetVariants(Vector<String> variants)
    {
        // update variants
        if(category_.equals("noun") == true)
        {
            catEntry_.GetNounEntry().SetVariants(variants);
        }
        else if(category_.equals("verb") == true)
        {
            catEntry_.GetVerbEntry().SetVariants(variants);
        }
        else if(category_.equals("adj") == true)
        {
            catEntry_.GetAdjEntry().SetVariants(variants);
        }
        else if(category_.equals("adv") == true)
        {
            catEntry_.GetAdvEntry().SetVariants(variants);
        }
        else if(category_.equals("pron") == true)
        {
            catEntry_.GetPronEntry().SetVariants(variants);
        }
        else if(category_.equals("modal") == true)
        {
            catEntry_.GetModalEntry().SetVariant(variants);
        }
        else if(category_.equals("aux") == true)
        {
            catEntry_.GetAuxEntry().SetVariant(variants);
        }
    }
    /**
    * Set the entire collection of annotations for this lexical record
    *
    * @param  annotations   the whole collection of annotations to be set
    */
    public void SetAnnotations(Vector<String> annotations)
    {
        annotations_ = annotations;
    }
    /**
    * Set signature for this lexical record
    *
    * @param  signature   the signature to be set
    */
    public void SetSignature(String signature)
    {
        signature_ = signature;
    }
    /**
    * Set the category for this lexical record
    *
    * @param  category   the name of category to be set
    */
    public void SetCat(String category)
    {
        category_ = category;
        catEntry_ = new CatEntry(category_);
    }
    /**
    * Set the category entry for this lexical record
    *
    * @param  catEntry   the category entry to be set
    */
    public void SetCatEntry(CatEntry catEntry)
    {
        catEntry_ = catEntry;
    }
    /**
    * Set the record end currly bracket for this lexical record
    *
    * @param  end   the record end currly bracket to be set
    */
    public void SetEnd(String end)
    {
        end_ = end;
    }
    /**
    * Set the flag to print out xml header
    *
    * @param  printXmlHeader   xml header printing flag
    */
    public void SetPrintXmlHeader(boolean printXmlHeader)
    {
        printXmlHeader_ = printXmlHeader;
    }
    /**
    * Set the xml header
    *
    * @param  xmlHeader   xml header
    */
    public void SetXmlHeader(String xmlHeader)
    {
        xmlHeader_ = xmlHeader;
    }
    /**
    * Set the xml root tag, lexRecords
    *
    * @param  xmlRootTag   xml root tag
    */
    public static void SetXmlRootTag(String xmlRootTag)
    {
        xmlRootTag_ = xmlRootTag;
    }
    /**
    * Get the category entry of this lexical record
    *
    * @return  category entry
    */
    public CatEntry GetCatEntry()
    {
        return catEntry_;
    }
    /**
    * Get xml header
    *
    * @return  xml header
    */
    public static String GetXmlHeader()
    {
        return xmlHeader_;
    }
    public static void main(String[] args)
    {
        LexRecord lexRec = new LexRecord();
        lexRec.SetBase("123");
        lexRec.SetCat("noun");
        lexRec.GetCatEntry().GetNounEntry().AddVariant("irreg|123||");
        lexRec.GetCatEntry().GetNounEntry().AddVariant("group(irreg|123|222|)");
        System.out.println("----------------");
        System.out.println(lexRec.GetText());
        System.out.println("----------------");
        InflVarsAndAgreements inflVars = lexRec.GetInflVarsAndAgreements();
        Vector<InflVar> inflValues = inflVars.GetInflValues();
        for(int i = 0; i < inflValues.size(); i++)
        {
            InflVar inflectionVar = inflValues.elementAt(i);
            System.out.println(inflectionVar.GetVar() + "|"
                + inflectionVar.GetCat() + "|"
                + inflectionVar.GetInflection() + "|"
                + inflectionVar.GetUnInfl() + "|"
                + inflectionVar.GetCit() + "|"
                + inflectionVar.GetType());
        }
    }
    // data member
    private String base_ = null;
    private Vector<String> spellingVars_ = new Vector<String>();
    private String eui_ = null;
    private String category_ = null;
    private CatEntry catEntry_ = null;
    private Vector<String> acronyms_ = new Vector<String>();
    private Vector<String> abbreviations_ = new Vector<String>();
    private Vector<String> annotations_ = new Vector<String>();
    private String signature_ = null;
    private String end_ = null;
    private boolean printXmlHeader_ = true;
    private static String xmlHeader_ = GlobalVars.GetXmlHeader();
    private static String xmlRootTag_ = "lexRecords";
}
