package gov.nih.nlm.nls.lexCheck.Lib;
import java.util.*;
/*****************************************************************************
* This class provides a Java object for verb entry.
*
* <p><b>History:</b>
* <ul>
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2013
****************************************************************************/
public class VerbEntry
{
    /**
    *    Creates a default verb entry object
    */
    public VerbEntry()
    {
        variants_ = new Vector<String>();
        intran_ = new Vector<String>();
        tran_ = new Vector<String>();
        ditran_ = new Vector<String>();
        link_ = new Vector<String>();
        cplxtran_ = new Vector<String>();
        nominalization_ = new Vector<String>();
    }
    // public method
    /**
    * Get the collection of inflectional variants of a verb entry
    *
    * @return  a Vector of inflectional variants of a verb entry.
    */
    public Vector<String> GetVariants()
    {
        return variants_;
    }
    /**
    * Get the collection of intransitive complements of a verb entry
    *
    * @return  a Vector of intransitive complements of a verb entry.
    */
    public Vector<String> GetIntran()
    {
        return intran_;
    }
    /**
    * Get the collection of transitive complements of a verb entry
    *
    * @return  a Vector of transitive complements of a verb entry.
    */
    public Vector<String> GetTran()
    {
        return tran_;
    }
    /**
    * Get the collection of ditransitive complements of a verb entry
    *
    * @return  a Vector of ditransitive complements of a verb entry.
    */
    public Vector<String> GetDitran()
    {
        return ditran_;
    }
    /**
    * Get the collection of linking complements of a verb entry
    *
    * @return  a Vector of linking complements of a verb entry.
    */
    public Vector<String> GetLink()
    {
        return link_;
    }
    /**
    * Get the collection of complex-transitive complements of a verb entry
    *
    * @return  a Vector of complex-transitive complements of a verb entry.
    */
    public Vector<String> GetCplxtran()
    {
        return cplxtran_;
    }
    /**
    * Get the collection of nominalization of a verb entry
    *
    * @return  a Vector of nominalization of a verb entry.
    */
    public Vector<String> GetNominalization()
    {
        return nominalization_;
    }
    /**
    * Add an inflectional variant to the collection of inflectional variants 
    * of a verb entry.
    *
    * @param  variant inflectional variant to be added.
    */
    public void AddVariant(String variant)
    {
        variants_.add(variant);
    }
    /**
    * Set the collection of inflectional variants of a verb entry
    *
    * @param  variants   a Vector of inflectional variants to be set.
    */
    public void SetVariants(Vector<String> variants)
    {
        variants_ = variants;
    }
    /**
    * Add an intransitive complement to the collection of intransitive 
    * complements of a verb entry.
    *
    * @param  intran    the intransitive complement to be added.
    */
    public void AddIntran(String intran)
    {
        intran_.add(intran);
    }
    /**
    * Add a transitive complement to the collection of transitive 
    * complements of a verb entry.
    *
    * @param  tran   the intransitive complement to be added.
    */
    public void AddTran(String tran)
    {
        tran_.add(tran);
    }
    /**
    * Add a ditransitive complement to the collection of ditransitive 
    * complements of a verb entry.
    *
    * @param  ditran   the ditransitive complement to be added.
    */
    public void AddDitran(String ditran)
    {
        ditran_.add(ditran);
    }
    /**
    * Add a linking complement to the collection of linking 
    * complements of a verb entry.
    *
    * @param  link   the linking complement to be added.
    */
    public void AddLink(String link)
    {
        link_.add(link);
    }
    /**
    * Add a complex-transitive complement to the collection of complex- 
    * complements of a verb entry.
    *
    * @param  cplxtran    the complex-transitive complement to be added.
    */
    public void AddCplxtran(String cplxtran)
    {
        cplxtran_.add(cplxtran);
    }
    /**
    * Set the collection of intransitive complement of a verb entry
    *
    * @param  intran   a Vector of intransitive complement to be set.
    */
    public void SetIntran(Vector<String> intran)
    {
        intran_ = intran;
    }
    /**
    * Set the collection of transitive complement of a verb entry
    *
    * @param  tran   a Vector of transitive complement to be set.
    */
    public void SetTran(Vector<String> tran)
    {
        tran_ = tran;
    }
    /**
    * Set the collection of ditransitive complement of a verb entry
    *
    * @param  ditran   a Vector of ditransitive complement to be set.
    */
    public void SetDitran(Vector<String> ditran)
    {
        ditran_ = ditran;
    }
    /**
    * Set the collection of linking complement of a verb entry
    *
    * @param  link   a Vector of linking complement to be set.
    */
    public void SetLink(Vector<String> link)
    {
        link_ = link;
    }
    /**
    * Set the collection of complex-transitive complement of a verb entry
    *
    * @param  cplxtran   a Vector of complex-transitive complement to be set.
    */
    public void SetCplxtran(Vector<String> cplxtran)
    {
        cplxtran_ = cplxtran;
    }
    /**
    * Add an nominalization to the collection of nominalizations 
    * of an verb entry.
    *
    * @param  nominalization nominalization String to be added into Vector.
    */
    public void AddNominalization(String nominalization)
    {
        nominalization_.add(nominalization);
    }
    /**
    * Set the collection of nominalizations of a verb entry
    *
    * @param  nominalization Vector of nominalization to be set.
    */
    public void SetNominalization(Vector<String> nominalization)
    {
        nominalization_ = nominalization;
    }
    /**
    * Get the text format of the verb entry
    *
    * @return  text format of the verb entry
    */
    public String GetText()
    {
        String text = new String();
        text = TextLib.AddToText(text, "variants=", variants_, 1);
        text = TextLib.AddToText(text, "", intran_, 1);
        text = TextLib.AddToText(text, "tran=", tran_, 1);
        text = TextLib.AddToText(text, "ditran=", ditran_, 1);
        text = TextLib.AddToText(text, "link=", link_, 1);
        text = TextLib.AddToText(text, "cplxtran=", cplxtran_, 1);
        text = TextLib.AddToText(text, "nominalization=", nominalization_, 1);
        return text;
    }
    /**
    * Get the xml format of the verb entry
    *
    * @return  xml format of the verb entry
    */
    public String GetXml()
    {
        boolean convertFlag = true;
        String xml = new String();
        xml = XmlLib.AddToXml(xml, "<verbEntry>", 2); 
        xml = XmlLib.AddToXml(xml, "<variants>", "</variants>", variants_, 3,
            convertFlag);
        xml = XmlLib.AddToXml(xml, "<intran>", "</intran>", intran_, 3,
            convertFlag);
        xml = XmlLib.AddToXml(xml, "<tran>", "</tran>", tran_, 3, convertFlag);
        xml = XmlLib.AddToXml(xml, "<ditran>", "</ditran>", ditran_, 3,
            convertFlag);
        xml = XmlLib.AddToXml(xml, "<link>", "</link>", link_, 3, convertFlag);
        xml = XmlLib.AddToXml(xml, "<cplxtran>", "</cplxtran>", cplxtran_, 3,
            convertFlag);
        xml = XmlLib.AddToXml(xml, "<nominalization>", "</nominalization>", 
            nominalization_, 3, convertFlag);
        xml = XmlLib.AddToXml(xml, "</verbEntry>", 2); 
        return xml;
    }
    // data member
    private Vector<String> variants_ = new Vector<String>();
    private Vector<String> intran_ = new Vector<String>();
    private Vector<String> tran_ = new Vector<String>();
    private Vector<String> ditran_ = new Vector<String>();
    private Vector<String> link_ = new Vector<String>();
    private Vector<String> cplxtran_ = new Vector<String>();
    private Vector<String> nominalization_ = new Vector<String>();
}
