package gov.nih.nlm.nls.lexCheck.Tools;
import java.util.*;
import java.io.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Gram.*;
import gov.nih.nlm.nls.lvg.CmdLineSyntax.*;
import gov.nih.nlm.nls.lvg.Util.*;
/*****************************************************************************
* This class provides a command line program for checking the grammer syntax 
* of a lexical record (text) read in from a file.  This program duplicates the
* work in Java class Api.ToXmlApi.ToXmlFromTextFile() with extra options can be 
* set.
*
* <p><b>History:</b>
* <ul>
* SCR-43, chlu, 07-11-13, add irregxpFile to config
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2013
****************************************************************************/
public class LexCheck extends SystemOption
{
    // public method
    /**
    * command line program for checking syntax of a lexical record.
    */
    public static void main(String[] args)
    {
        String optionStr = new String();
        Option inOption = new Option(optionStr);
        
        // form optin String if there are arguments
        if(args.length > 0)
        {
            optionStr = new String();
            for(int i = 0; i < args.length; i++)
            {
                if(i == 0)
                {
                    optionStr = args[i];
                }
                else
                {
                    optionStr += (" " + args[i]);
                }
            }
            inOption = new Option(optionStr);
        }
        // define the system option flag & argument
        LexCheck lexCheck = new LexCheck();
        Vector<OptionItem> optionItems = inOption.GetOptionItems();
        // execute command according to option & argument
        if(SystemOption.CheckSyntax(inOption, lexCheck.GetOption(), false, 
            true) == true)
        {
            // decode input option to form options
            lexCheck.ExecuteCommands(inOption, lexCheck.GetOption());
            // check if there is input file specified
            if((runFlag_ == true) && (inReader_ == null)) 
            {
                try
                {
                    helpMenu();
                }
                catch (Exception e) { }
            }
            else if(runFlag_ == true)
            {
                // get config file from environment variable
                boolean useClassPath = false;
                if(configFile_ == null)
                {
                    useClassPath = true;
                    configFile_ = "data.config.lexCheck";
                }
                // read in configuration file
                if(conf_ == null)
                {
                    conf_ = new Configuration(configFile_, useClassPath);
                }
                // read in input text and get the Norm for it
                RunProgram();
            }
        }
        else
        {
            helpMenu();
        }
        // clsoe open files
        try
        {
            Close();
        }
        catch (Exception e)
        {
        }
    }
    // read data from user's line inputs
    private static void RunProgram()
    {
        // set the text and Xml indent from the configuration file
        String ti = conf_.GetConfiguration(Configuration.TEXT_INDENT);
        String xi = conf_.GetConfiguration(Configuration.XML_INDENT);
        String xmlHeader = conf_.GetConfiguration(Configuration.XML_HEADER);
        String prepositionFile 
            = conf_.GetConfiguration(Configuration.LC_DIR)
            + conf_.GetConfiguration(Configuration.PREPOSITION_FILE);
        String irregExpFile 
            = conf_.GetConfiguration(Configuration.LC_DIR)
            + conf_.GetConfiguration(Configuration.IRREG_EXP_FILE);
        GlobalVars.SetTextIndent(GetStringContent(ti));
        GlobalVars.SetXmlIndent(GetStringContent(xi));
        GlobalVars.SetXmlHeader(GetStringContent(xmlHeader));
        GlobalVars.SetPrepositionFile(GetStringContent(prepositionFile));
        GlobalVars.SetIrregExpFile(GetStringContent(irregExpFile));
        CheckRecord();
    }
    private static String GetStringContent(String inStr)
    {
        int length = inStr.length();
        if((inStr != null)
        && (inStr.startsWith("\"") == true)
        && (inStr.endsWith("\"") == true))
        {
            return inStr.substring(1, length-1);
        }
        return inStr;
    }
    protected void ExecuteCommand(OptionItem optionItem, Option systemOption)
    {
        OptionItem nameItem =
            OptionUtility.GetItemByName(optionItem, systemOption, false);
        
        if(CheckOption(nameItem, "-d") == true)        // debug message
        {
            debugFlag_ = true;
        }
        else if(CheckOption(nameItem, "-f:r") == true)     // text release output
        {
            format_ = RELEASE_FORMAT;
        }
        else if(CheckOption(nameItem, "-f:t") == true)     // text output
        {
            format_ = TEXT_FORMAT;
        }
        else if(CheckOption(nameItem, "-f:tx") == true) // text and XML output
        {
            format_ = TEXT_XML_FORMAT;
        }
        else if(CheckOption(nameItem, "-f:x") == true)     // xml output
        {
            format_ = XML_FORMAT;
        }
        else if(CheckOption(nameItem, "-f:nxh") == true) // no xml header
        {
            printXmlHeader_ = false;
        }
        else if(CheckOption(nameItem, "-h") == true)
        {
            helpMenu();
            runFlag_ = false;
        }
        else if(CheckOption(nameItem, "-hs") == true)
        {
            systemOption.PrintOptionHierachy();        // not UTF-8
            runFlag_ = false;
        }
        else if(CheckOption(nameItem, "-i:STR") == true)
        {
            String inFile = nameItem.GetOptionArgument();
            if(inFile != null)
            {
                try
                {
                    inReader_ = new BufferedReader(new InputStreamReader(
                        new FileInputStream(inFile), "UTF-8"));
                }
                catch (IOException e)
                {
                    runFlag_ = false;
                    System.err.println(
                        "**Error: problem of opening/reading file " + inFile);
                }
            }
        }
        else if(CheckOption(nameItem, "-o:STR") == true)
        {
            String outFile = nameItem.GetOptionArgument();
            try
            {
                outWriter_ = new BufferedWriter(new OutputStreamWriter(
                    new FileOutputStream(outFile), "UTF-8"));
                fileOutput_ = true;
            }
            catch (IOException e)
            {
                runFlag_ = false;
                System.err.println(
                    "**Error: problem of opening/writing file " + outFile);
            }
        }
        else if(CheckOption(nameItem, "-v") == true)
        {
            try
            {
                 Out.Println(outWriter_, "lexCheck.2013", fileOutput_, false);
            }
            catch (IOException e) { }
            runFlag_ = false;
        }
        else if(CheckOption(nameItem, "-x:STR") == true)
        {
            configFile_ = nameItem.GetOptionArgument();
        }
    }
    private static void helpMenu()
    {
        // print out the usage
        MenuPrint("");
        MenuPrint("Synopsis:");
        MenuPrint("  LexCheck [options]");
        MenuPrint("");
        MenuPrint("Description:");
        MenuPrint("  Validates text syntax in LexRecords and convert them among text, Xml, and Java objects from an input file.");
        MenuPrint("");
        MenuPrint("Options:");
        MenuPrint("  -d:     Print syntax validation debug message");
        MenuPrint("  -f:r    Print lexical records in release format");
        MenuPrint("  -f:t    Print lexical records in text format");
        MenuPrint("  -f:tx   Print lexical records in text and Xml format");
        MenuPrint("  -f:x    Print lexical records in Xml format");
        MenuPrint("  -f:nxh  No Xml header with version information");
        MenuPrint("  -h      Print program help information (this is it).");
        MenuPrint("  -hs     Print option's hierarchy structure.");
        MenuPrint("  -i:STR  Define input file");
        MenuPrint("  -o:STR  Define output file");
        MenuPrint("  -v:     Return the current verion identification of LexCheck");
        MenuPrint("  -x:STR  Loading an alternative configuration file.");
    }
    private static void MenuPrint(String text)
    {
        try
        {
            Out.Println(outWriter_, text, fileOutput_, false);
        }
        catch (Exception e)
        {
        }
    }
    protected void DefineFlag()
    {
        // define all option flags & arguments by giving a option string
        String flagStr = "-d -f:r:t:tx:x:nxh -h -hs -i:STR -o:STR -v -x:STR";
        // init the system option
        systemOption_ = new Option(flagStr);
        // Add the full name for flags
        systemOption_.SetFlagFullName("-d", "Print_Error_Msg");
        systemOption_.SetFlagFullName("-f:r", "Release_Format");
        systemOption_.SetFlagFullName("-f:t", "Text_Format");
        systemOption_.SetFlagFullName("-f:tx", "Text_Xml_Format");
        systemOption_.SetFlagFullName("-f:x", "Xml_Format");
        systemOption_.SetFlagFullName("-f:nxh", "No_Xml_Header");
        systemOption_.SetFlagFullName("-h", "Help");
        systemOption_.SetFlagFullName("-hs", "Hierarchy_Struture");
        systemOption_.SetFlagFullName("-i", "Input_File");
        systemOption_.SetFlagFullName("-o", "Output_File");
        systemOption_.SetFlagFullName("-v", "Version");
        systemOption_.SetFlagFullName("-x", "Load_Configuration_file");
    }
    private static void Close() throws IOException
    {
        if(outWriter_ != null)
        {
            outWriter_.close();
        }
        if(inReader_ != null)
        {
            inReader_.close();
        }
    }
    // check input line by line
    private static void CheckRecord() 
    {
        try
        {
            // add header and root beign tag
            if(format_ == XML_FORMAT) 
            {
                if(printXmlHeader_ == true)
                {
                    Out.Println(outWriter_, LexRecord.GetXmlHeader(),
                        fileOutput_, false);
                }
                Out.Println(outWriter_, LexRecord.GetXmlRootBeginTag(),
                    fileOutput_, false);
            }
            while(true)
            {
                // check if read in the next line
                if(lineObject_.IsGoToNext() == true)
                {
                    lineObject_.SetLine(inReader_.readLine());
                    lineObject_.IncreaseLineNum();
                }
                if(CheckLine() == false)
                {
                    break;
                }
            }
            // add root end tag
            if(format_ == XML_FORMAT) 
            {
                Out.Println(outWriter_, LexRecord.GetXmlRootEndTag(),
                    fileOutput_, false);
            }
            inReader_.close();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }
    // check grammer for the read-in line 
    private static boolean CheckLine() 
    {
        boolean flag = false;
        // check input for quitiing
        if((lineObject_ == null) || (lineObject_.GetLine() == null))
        {
            return false;
        }
        else
        {
            flag = CheckGrammer.Check(lineObject_, printFlag_, st_, catSt_,
                debugFlag_);
            // print out lex record 
            if((st_.GetCurState() == CheckSt.CHECK_BASE)
            && (flag == true))
            {
                try
                {
                    switch(format_)
                    {
                        case RELEASE_FORMAT:
                            Out.Print(outWriter_, 
                                CheckGrammer.GetLexRecord().GetReleaseFormatText(), 
                                fileOutput_, false);
                            break;
                        case TEXT_FORMAT:
                            Out.Print(outWriter_, 
                                CheckGrammer.GetLexRecord().GetText(), 
                                fileOutput_, false);
                            break;
                        case TEXT_XML_FORMAT:
                            Out.Println(outWriter_, 
                                CheckGrammer.GetLexRecord().GetText(),
                                fileOutput_, false);
                            Out.Println(outWriter_, 
                                CheckGrammer.GetLexRecord().GetXml(0),
                                fileOutput_, false);
                            break;
                        case XML_FORMAT:
                            Out.Println(outWriter_, 
                                CheckGrammer.GetLexRecord().GetXml(1),
                                fileOutput_, false);
                            break;
                    }
                }
                catch (Exception e) { }
            }
        }
        return flag;
    }
    // data member
    private static BufferedReader inReader_ =  null;        // inFile buffer
    private static BufferedWriter outWriter_ = null;        // outFile buffer
    private static boolean fileOutput_ = false;        // flag for file output
    private static boolean runFlag_ = true;         // flag for running lexCheck
    private static boolean debugFlag_ = false;
    private static boolean printFlag_ = true;
    private static boolean printXmlHeader_ = true;
    private final static int TEXT_FORMAT = 0;
    private final static int XML_FORMAT = 1;
    private final static int TEXT_XML_FORMAT = 3;
    private final static int RELEASE_FORMAT = 4;
    private static int format_ = TEXT_FORMAT;
    private static String cDate = new String();
    private static String configFile_ = null;
    private static Configuration conf_ = null;
    private static CheckSt st_ = new CheckSt();        // upper level  Check state
    private static CheckSt catSt_ = new CheckSt(CheckSt.CHECK_CAT_ENTRY);
        // Category dependant Check state
    private static LineObject lineObject_ = new LineObject();
    static
    {
        try
        {
             outWriter_ = new BufferedWriter(new OutputStreamWriter(
                 System.out, "UTF-8"));
        }
        catch(Exception e)
        {
            System.out.println("** Error: File Exception: " + e.toString());
        }
        
    }
}
