package gov.nih.nlm.nls.lexCheck.Tools;
import com.ibm.xml.parser.Parser;
import java.io.*;
import java.util.*;
import gov.nih.nlm.nls.lexCheck.Api.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.CheckCont.*;
/*****************************************************************************
* This class provides a command line program for Lexicon cross check.
* It cross checks lexRecord, EUIs, and cross-ref EUIs.
*
* <p><b>History:</b>
* <ul>
* <li>SCR-40, chlu, 04/04/2013, Add new features to check Lexicon
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2013
****************************************************************************/
public class LexCrossCheck
{
    /**
    * command line program for validating lexical records' base and 
    * spelling variatns with variants setting (glreg & regd)
    * by reading a Text foramt from a input file.
    */
    static public void main(String[] argv) 
    {
        String inFile = "LEXICON";
        String autoFixFile = "LEXICON.fix";
		boolean verbose = false;
		String prepositionFile = "../data/Files/prepositions.data";
		String dupRecExpFile = "../data/Files/dupRecExceptions.data";
		String notBaseFormFile = "../data/Files/notBaseForm.data";

		if(argv.length == 6)
        {
            inFile = argv[0];
            autoFixFile = argv[1];
            if(argv[2].equals("-v") == true)
            {
                verbose = true;
            }
            prepositionFile = argv[3];
            dupRecExpFile = argv[4];
			notBaseFormFile = argv[5];
        }
        else if(argv.length == 3)
        {
            inFile = argv[0];
            autoFixFile = argv[1];
			if(argv[2].equals("-v") == true)
			{
				verbose = true;
			}
        }
        else if(argv.length == 2)
        {
            inFile = argv[0];
            autoFixFile = argv[1];
        }
        else if(argv.length == 1)
        {
            inFile = argv[0];
			autoFixFile = inFile + ".fix";
        }
        else if(argv.length >= 0)
        {
            System.err.println("** Usage: java LexCrossCheck <inFile)> <outFile> <-v> <prepositionFile> <dupRecExpFile> <notBaseFormFile>");
            System.err.println("");
            System.err.println("Options:");
            System.err.println("  inFile: LEXICON in text format");
            System.err.println("  outFile: auto-fixed LEXICON in text format");
            System.err.println("  -v: verbose");
			System.err.println("  prepositionFile: preposition file");
			System.err.println("  dupRecExpFile: dup record exception file");
			System.err.println("  notBaseFormFile: not base form file");
            System.exit(1);
        }
		
		// verbose
		if(verbose == true)
		{
			System.out.println("===== LexCrossCheck( ) =====");
			System.out.println("-- inFile: " + inFile);
			System.out.println("-- autoFixFile: " + autoFixFile);
			System.out.println("-- prepositionFile: " + prepositionFile);
			System.out.println("-- dupRecExpFile: " + dupRecExpFile);
			System.out.println("-- notBaseFormFile: " + notBaseFormFile);
		}

		// set up preposition
		GlobalVars.SetPrepositionFile(prepositionFile);

		// read in dupExplist
		int lineNum = 500;
		Hashtable<String, HashSet<String>> dupRecExpList
			= KeyValuesTable.GetKey2ValuesTable(dupRecExpFile, verbose, 
			lineNum);

		// read in notBaseFormSet
		boolean useDefaultFileFlag = false; 
		HashSet<String> notBaseFormSet
			= SetTable.GetSetTabe(notBaseFormFile, verbose, lineNum,
			useDefaultFileFlag);
		
        // check contents from a inFile LEXICON
        CrossCheckLexRecords.CheckFromFile(inFile, autoFixFile, verbose, 
			dupRecExpList, notBaseFormSet);
    }
}
