package gov.nih.nlm.nls.lexCheck.Tools;
import com.ibm.xml.parser.Parser;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Hashtable;
import org.w3c.dom.*;
import java.util.*;
import gov.nih.nlm.nls.lexCheck.Api.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lvg.Lib.*;
/*****************************************************************************
* This class provides a command line program for generating java object of 
* a lexical record which reads in it's Xml format from a file.
*
* <p><b>History:</b>
* <ul>
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2013
****************************************************************************/
public class ToJavaObjectFromXmlFile
{
    /**
    * command line program for generating java object of a lexical record 
    * by reading a Xml foramt from a input file.
    */
    static public void main(String[] argv) 
    {
        if((argv.length == 0) || (argv.length > 2))
        {
             System.err.println("** Usage: java ToJavaObjectFromXmlFile <inFile(Xml)> <-i>");
            System.err.println("");
            System.err.println("Options:");
            System.err.println("  -i: generate inflection vars");
            System.exit(1);
        }
        boolean inflVarFlag = false;
        for(int i = 0; i < argv.length; i++)
        {
            if(argv[i].equals("-i"))    // infl vars
            {
                inflVarFlag = true;
                break;
            }
        }
        try
        {
            Vector<LexRecord> lexRecords 
                = ToJavaObjApi.ToJavaObjsFromXmlFile(argv[0]);
            if(lexRecords.size() <= 0)
            {
                System.exit(1);
            }
            else
            {
                for(int i = 0; i < lexRecords.size(); i++)
                {
                    LexRecord lexRecord = lexRecords.elementAt(i);
                    System.out.print(lexRecord.GetText());
                    if(inflVarFlag == true)
                    {
                    System.out.println("---------- Inflection Vars ----------");
                    Vector<InflVar> inflVars = 
                        lexRecord.GetInflVarsAndAgreements().GetInflValues();
                    for(int j = 0; j < inflVars.size(); j++)
                    {
                        InflVar inflVar = inflVars.elementAt(j);
                        System.out.println(inflVar.GetVar() + "|"
                            + Category.ToValue(inflVar.GetCat()) + "|"
                            + Inflection.ToValue(inflVar.GetInflection()) + "|"
                            + inflVar.GetEui() + "|"
                            + inflVar.GetUnInfl() + "|"
                            + inflVar.GetCit());
                    }
                    }
                }
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }
}
