package gov.nih.nlm.nls.lexCheck.Tools;
import com.ibm.xml.parser.Parser;
import java.io.*;
import java.util.Hashtable;
import org.w3c.dom.*;
import java.util.*;
import gov.nih.nlm.nls.lexCheck.Api.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.CheckCont.*;
/*****************************************************************************
* This class provides a command line program for validating contents.
* It validates all lexical records locally (no cross-reference check).
* It also provides option to fix the found issues automatically.
* It includes base and spelling variatns ending with variants (glreg & regd) 
* which reads in it's Text format from a file.
*
* <p><b>History:</b>
* <ul>
* <li>SCR-39, chlu, 03/28/2013, Add new features to check content
* <li>SCR-44, chlu, 07/12/2013, use HashSet to pass irregExpEuiList
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2013
****************************************************************************/
public class ValidateContentFromTextFile
{
    /**
    * command line program for validating lexical records' base and 
    * spelling variatns with variants setting (glreg & regd)
    * by reading a Text foramt from a input file.
    */
    static public void main(String[] argv) 
    {
        String inFile = "LEXICON";
        String outFile = "LEXICON.fix";
		String prepositionFile = "../data/Files/prepositions.data";
		String irregExpFile = "../data/Files/irregExceptions.data";
		boolean verbose = false;
        if(argv.length == 1)
        {
            inFile = argv[0];
			outFile = inFile + ".fix";
        }
        else if(argv.length == 2)
        {
            inFile = argv[0];
            outFile = argv[1];
        }
        else if(argv.length == 3)
        {
            inFile = argv[0];
            outFile = argv[1];
			if(argv[2].equals("-v") == true)
			{
				verbose = true;
			}
        }
        else if(argv.length == 5)
        {
            inFile = argv[0];
            outFile = argv[1];
			if(argv[2].equals("-v") == true)
			{
				verbose = true;
			}
			prepositionFile = argv[3];
			irregExpFile = argv[4];
        }
        else if(argv.length >= 0)
        {
            System.err.println("** Usage: java ValidateContentFromTextFile <inFile(Text)> <outFile> <-v> <prepositionFile> <irregExpFile>");
            System.err.println("");
            System.err.println("Options:");
            System.err.println("  inFile: LEXICON in text format");
            System.err.println("  outFile: auto-fixed LEXICON in text format");
            System.err.println("  -v: verbose");
            System.err.println("  prepositionFile: preposition file");
            System.err.println("  irregExpFile: irreg exception EUI file");
            System.exit(1);
        }

		if(verbose == true)
		{
			System.out.println("===== ValidateContentFromTextFile( ) =====");	
			System.out.println("-- inFile: " + inFile);
			System.out.println("-- prepositionFile: " + prepositionFile);
			System.out.println("-- irregExpFile: " + irregExpFile);
		}
		// set preposition
		GlobalVars.SetPrepositionFile(prepositionFile);
		
		// get the irregExpEuiList
		int lineNum = 100;
		HashSet<String> irregExpEuiList 
			= CheckVariants.GetIrregExpEuiListFromFile(irregExpFile, verbose,
			lineNum);
		 
        // check contents from a inFile LEXICON
        CheckLexRecord.CheckLexRecordsFromFile(inFile, outFile, false, 
			irregExpEuiList);
    }
}
